#!/usr/bin/env python
# -*- coding: utf-8 -*-

###########################################################################
# Wake on LAN.
#
# Copyright (C) 2010-2013 Alkis Georgopoulos <alkisg@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FINESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# On Debian GNU/Linux systems, the complete text of the GNU General
# Public License can be found in `/usr/share/common-licenses/GPL".
###########################################################################

import sys
import socket
import struct
import netifaces

def getBroadcastList():
    brlist = ['<broadcast>']
    ifaces = [iface for iface in netifaces.interfaces() if iface != 'lo']
    for ifname in ifaces:
        # An interface can have more than one address, even within the 
        # same family (AF_INET), or none, so check this case too.
        addresses = netifaces.ifaddresses(ifname)
        if netifaces.AF_INET not in addresses:
            continue
        for addr in addresses[netifaces.AF_INET]:
            if 'broadcast' in addr:
                brlist.append(addr['broadcast'])
    return brlist


def wake_on_lan(macaddress):
    """ Switches on remote computers using WOL. """

    # Check macaddress format and try to compensate.
    if len(macaddress) == 12:
        pass
    elif len(macaddress) == 12 + 5:
        sep = macaddress[2]
        macaddress = macaddress.replace(sep, '')
    else:
        raise ValueError('Incorrect MAC address format')
 
    print "Sending magic packet to", macaddress
    # Pad the synchronization stream.
    data = ''.join(['FFFFFFFFFFFF', macaddress * 20])
    send_data = '' 

    # Split up the hex values and pack.
    for i in range(0, len(data), 2):
        send_data = ''.join([send_data,
                             struct.pack('B', int(data[i: i + 2], 16))])

    # Broadcast it to the LAN.
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
    for br in getBroadcastList():
        sock.sendto(send_data, (br, 9))

if __name__ == '__main__':
    for mac in sys.argv[1:]:
        wake_on_lan(mac)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###########################################################################
# Notifications.
#
# Copyright (C) 2015 Fotis Tsamis <ftsamis@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FINESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# On Debian GNU/Linux systems, the complete text of the GNU General
# Public License can be found in `/usr/share/common-licenses/GPL".
###########################################################################

import pynotify
import os

if not pynotify.init("epoptes-notifications"):
    sys.exit(1)


class NotificationCache(object):
    """A class to store notifications by 'summary' until they get closed/hidden.
    This helps to emulate the x-canonical-append capability on systems that
    do not provide it.
    """
    def __init__(self):
        self.notifications = {}

    def add_notification(self, n, title):
        # The title argument is important to handle unicode summaries correctly.
        # n.props.summary does not return unicode objects.
        self.notifications[title] = n
        n.connect("closed", self.remove_notification, title)

    def get_notification(self, title):
        if title in self.notifications:
            return self.notifications[title]

    def remove_notification(self, n, title):
        # The title argument is important to handle unicode summaries correctly.
        del self.notifications[title]


def notify(title, body, icon):
    n = pynotify.Notification(title, body, icon)
    n.set_hint_string("x-canonical-append", "true")
    n.show()


def cached_notify(title, body, icon):
    """A function to replace pynotify.notify if the 'x-canonical-append'
    capability is not provided.
    """
    n = cache.get_notification(title)
    if n:
        n.close()
        n.update(title, n.props.body+'\n'+body, icon)
    else:
        n = pynotify.Notification(title, body, icon)
        cache.add_notification(n, title)
    n.show()


append = 'x-canonical-append' in pynotify.get_server_caps()
if not append:
    cache = NotificationCache()
    notify = cached_notify

def shutdownNotify(host):
    notify(_("Shut down:"), "%s" %(host), os.path.abspath("images/shutdown.svg"))

def loginNotify(user, host):
    notify(_("Connected:"), _("%(user)s on %(host)s") %{"user":user, "host":host},
                                os.path.abspath("images/login.svg"))
def logoutNotify(user, host):
    notify(_("Disconnected:"), _("%(user)s from %(host)s") %{"user":user, "host":host},
                                os.path.abspath("images/logout.svg"))




#!/usr/bin/env python
# -*- coding: utf-8 -*-

###########################################################################
# Configuration file parser and other default configuration variables.
#
# Copyright (C) 2011 Alkis Georgopoulos <alkisg@gmail.com>
# Copyright (C) 2011 Fotis Tsamis <ftsamis@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FINESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# On Debian GNU/Linux systems, the complete text of the GNU General
# Public License can be found in `/usr/share/common-licenses/GPL".
###########################################################################

import os
import shlex
import ConfigParser
import json
import gettext
gettext.install('epoptes', unicode=True)
import locale
locale.textdomain('epoptes')

from epoptes.core import structs
from epoptes.common.constants import *

def read_plain_file(filename):
    """Return the whole contents of a plain text file into a string list.

    If the file doesn't exist or isn't readable, return an empty list.
    """
    
    try:
        f = open(filename, 'r')
        contents = [x.strip() for x in f.readlines()]
        f.close()
        return contents
    except:
        return []


def write_plain_file(filename, contents):
    """Write the contents string list to filename. Return True if successful.
    """

    try:
        if not os.path.isdir(path):
            os.makedirs(path)
        f = open(filename, 'w')
        f.write('\n'.join(contents))
        return True
    except:
        return False


def read_ini_file(filename):
    """Return a ConfigParser from the contents of a configuration file.
    """
    conf = ConfigParser.ConfigParser()
    try:
        conf.read(filename)
    except:
        pass
    return conf


def write_ini_file(filename, contents):
    """Write contents to a ConfigParser file. Return True if successful.
    """
    conf = contents
    try:
        conf.write(filename)
        return True
    except:
        return False


def read_shell_file(filename):
    """Return the variables of a shell-like configuration file in a dict.

    If the file doesn't exist or isn't readable, return an empty list.
    Also strip all comments, if any.
    """
    
    if not os.path.isfile(filename):
        return {}
    try:
        f = open(filename, 'r')
        contents = f.read()
        f.close()
        contents = shlex.split(contents, True)
        # TODO: maybe return at least all the valid pairs?
        return dict(v.split('=') for v in contents)
    except:
        return {}

def read_groups(filename):
    """Parse a JSON file and create the appropriate group and
    client objects.
    
    Return a 2-tuple with a client instances list and a group
    instances list.
    """
    try:
        f=open(filename)
        data = json.loads(f.read())
        f.close()
    except:
        return ([],[])
    
    saved_clients = {}

    for key, cln in data['clients'].iteritems():
        new = structs.Client('offline', cln['mac'], '', cln['alias'])
        saved_clients[key] = new

    groups = []
    for grp in data['groups']:
        members = {}
        for key, dct in grp['members'].iteritems():
            members[saved_clients[key]] = dct
    
        groups.append(structs.Group(grp['name'], members))
    
    return (saved_clients.values(), groups)
    
def save_groups(filename, model):
    """Save the groups and their members from model (gtk.ListStore)
    in JSON format.
    """
    path = os.path.expanduser('~/.config/epoptes/')
    try:
        if not os.path.isdir(path):
            os.makedirs(path)
    except:
        pass
    
    data = {'clients' : {}, 'groups' : []}
    uid=0
    uid_pairs = {}
    saved_clients = []
    
    # Create a list with all the clients we want to save
    for grp in model:
        grp = grp[G_INSTANCE]
        for cln in grp.get_members():
            if cln not in saved_clients:
                saved_clients.append(cln)
    
    for cln in saved_clients:
        # Use an integer ID as a key instead of the memory address
        data['clients'][uid] = {'mac' : cln.mac, 'alias' : cln.alias}
        # Pair memory addresses with integer IDs
        uid_pairs[cln] = uid
        uid += 1

    for grp in model:
        grp = grp[G_INSTANCE]
        members = {}
        
        # Get the IDs created above
        for cln, props in grp.members.iteritems():
            members[uid_pairs[cln]] = props
        
        
        data['groups'].append({'name' : grp.name, 
                               'members' : members})
        
    # Save the dict in JSON format
    try:
        f=open(filename, 'w')
        f.write(json.dumps(data, indent=2))
        f.close()
    except:
        pass

def write_history():
    write_plain_file(os.path.join(path, 'history'), history)


# The system settings are shared with epoptes-clients, that's why the caps.
system = read_shell_file('/etc/default/epoptes')
# TODO: check if the types, e.g. PORT=int, may cause problems.
system.setdefault('PORT', 789)
system.setdefault('SOCKET_GROUP', 'epoptes')
system.setdefault('DIR', '/var/run/epoptes')
# Allow running unencrypted, for clients with very low RAM.
try:
    if os.path.getsize('/etc/epoptes/server.crt') == 0:
        system.setdefault('ENCRYPTION', False)
except:
    pass
finally:
    system.setdefault('ENCRYPTION', True)


path = os.path.expanduser('~/.config/epoptes/')
settings_file = os.path.join(path, 'settings')
settings = read_ini_file(settings_file)
if not settings.has_section('GUI'):
    settings.add_section('GUI')
user = {}
if settings.has_option('GUI', 'thumbnails_width'):
    user['thumbnails_width'] = settings.getint('GUI', 'thumbnails_width')
if settings.has_option('GUI', 'thumbnails_height'):
    user['thumbnails_height'] = settings.getint('GUI', 'thumbnails_height')
if not settings.has_option('GUI', 'messages_default_title'):
    settings.set('GUI', 'messages_default_title', _('Message from administrator'))
if not settings.has_option('GUI', 'messages_use_markup'):
    settings.set('GUI', 'messages_use_markup', 'False')
if not settings.has_option('GUI', 'grabkbdptr'):
    settings.set('GUI', 'grabkbdptr', 'False')

history = read_plain_file(os.path.join(path, 'history'))

# For debugging reasons, if ran from command line, dump the config
if __name__ == '__main__':
    print system




#!/usr/bin/env python
# -*- coding: utf-8 -*-

###########################################################################
# GUI plex.
#
# Copyright (C) 2010 Fotis Tsamis <ftsamis@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FINESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# On Debian GNU/Linux systems, the complete text of the GNU General
# Public License can be found in `/usr/share/common-licenses/GPL".
###########################################################################

import os
import tempfile

from twisted.internet import protocol
from twisted.protocols import amp

import commands
import exchange

class GUI(amp.AMP):

    def connectionMade(self):
        amp.AMP.connectionMade(self)
        exchange.knownGUIs.append(self)


    def connectionLost(self, reason):
        amp.AMP.connectionLost(self, reason)
        exchange.knownGUIs.remove(self)


    def clientConnected(self, handle):
        self.callRemote(commands.ClientConnected, handle=handle)


    def clientDisconnected(self, handle):
        self.callRemote(commands.ClientDisconnected, handle=handle)


    @commands.EnumerateClients.responder
    def enumerateClients(self):
        return {'handles': sorted(exchange.knownClients.iterkeys())}


    @commands.ClientCommand.responder
    def clientCommand(self, handle, command):
        if handle not in exchange.knownClients:
            print " Unknown client %s, can't execute %s" %(handle,command)
            #raise ValueError("Unknown client")
            return {'filename': '', 'result': ''}

        d = exchange.knownClients[handle].command(command.encode("utf-8"))
        
        def sendResult(result):
            if len(result) > 65000:
                tf = tempfile.NamedTemporaryFile('wb', dir="/var/run/epoptes", delete=False)
                tf.write(result)
                os.fchmod(tf.file.fileno(), 0660)
                return {'filename': tf.name, 'result': ''}
            else:
                return {'result': result, 'filename': ''}

        d.addCallback(sendResult)
        return d


class GUIFactory(protocol.ServerFactory):
    protocol = GUI



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###########################################################################
# Imports/exports user accounds from/to various sources.
#
# Copyright (C) 2009 Τεχνική Στήριξη ΣΕΠΕΗΥ <devs@ts.sch.gr>
#  * Original: Sep 2009 - Alkis Georgopoulos <alkisg@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FINESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# On Debian GNU/Linux systems, the complete text of the GNU General
# Public License can be found in `/usr/share/common-licenses/GPL".
###########################################################################

import pwd
import spwd
import grp
import operator
import csv
import subprocess
import re
import crypt
import random

# I split pw_gecos to rname, office, wphone, hphone
# I also use an additional field called plainpw, for plain text passwords.


DEFAULT_GROUPS = ["cdrom", "floppy", "dialout", "tape", "dip", "adm", "plugdev",
    "fax", "fuse", "video"]
FIELD_NAMES = ["Όνομα χρήστη", "UID", "Ονοματεπώνυμο", "Γραφείο", 
    "Τηλ. εργασίας", "Τηλ. οικίας", "Προσωπικός φάκελος", "Κέλυφος", "Ομάδες",
    "Ελάχιστη διάρκεια κωδικού", "Μέγιστη διάρκεια κωδικού", 
    "Ημέρες προειδοποίησης λήξης", "Ημέρες απενεργοποίησης μετά τη λήξη",
    "Ημέρες κλειδωμένου κωδικού", "Κρυπτογραφημένος κωδικός", 
    "Κωδικός πρόσβασης"]

UID_MIN = 1000  # TODO: the callee should read them from /etc/adduser.conf
UID_MAX = 60000 # and pass them to the command line.

class User:
    def __init__(self, name, uid='', rname='', office='', wphone='',
                 hphone='', dir='', shell='', groups=[], min='', max='',
                 warn='', inact='', expire='', pwd='', plainpw=''):

        self.name, self.uid, self.rname, self.office, self.wphone, self.hphone,\
        self.dir, self.shell, self.groups, self.min, self.max, self.warn, \
        self.inact, self.expire, self.pwd, self.plainpw = name, uid, rname, \
        office, wphone, hphone, dir, shell, groups, min, max, warn, inact, \
        expire, pwd, plainpw

def run_command(cmdline):
    # Runs a command and returns either an empty string, on successful
    # completion, or the whole stdout and stderr of the command, on error.

    # Popen doesn't like integers like uid or gid in the command line.
    cmdline = [str(s) for s in cmdline]

    p = subprocess.Popen(cmdline, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    res = p.wait()
    if res == 0:
        return ""
    else:
        print "Σφάλμα κατά την εκτέλεση εντολής:"
        print " $ %s" % ' '.join(cmdline)
        print p.stdout.read()
        err = p.stderr.read()
        print err
        if err == '':
            err = "\n"
        return err


def sorted_users(users):
    # TODO
    return users

def read_users_from_passwd(dirname="/etc"):
    """
    Reads users from /etc/passwd, /etc/shadow (if it has access) and /etc/group
    """
    pwds = pwd.getpwall()
    spwds = spwd.getspall()
    sn = {}
    for s in spwds:
        sn[s.sp_nam] = s
    users = {}

    for p in pwds:
        if p.pw_uid >= UID_MIN and p.pw_uid <= UID_MAX:
            if p.pw_name in sn:
                s = sn[p.pw_name]
            else:
                #print " * I couldn't find user %s in shadow file. Are you \
#root?" % p.pw_name
                s = spwd.struct_spwd(["", "x", "", "", "", "", "", "", ""])
            rname, office, wphone, hphone = (p.pw_gecos + ",,,").split(",")[:4]
            u = User(p.pw_name, p.pw_uid, rname, office, wphone, hphone,
                p.pw_dir, p.pw_shell, [], s.sp_min, s.sp_max, s.sp_warn, 
                s.sp_inact, s.sp_expire, s.sp_pwd, "")
            if u.inact == -1:
                u.inact = ''
            if u.expire == -1:
                u.expire = ''
            users[u.name] = u

    grps = grp.getgrall()
    for g in grps:
        for gu in g.gr_mem:
            if gu in users:
                users[gu].groups.append(g.gr_name)

    return sorted_users(users)


def print_user(u):
    """
    Prints a user to stdout, mainly for debugging
    """
    print u.name, u.uid, u.rname, u.office, u.wphone, u.hphone, u.dir, u.shell,\
        u.groups, u.min, u.max, u.warn, u.inact, u.expire, u.pwd, u.plainpw


def print_users(users):
    """
    Prints all the users to stdout, mainly for debugging
    """
    for name, u in users.iteritems():
        print_user(u)


def export_users_to_csv(users, filename):
    """
    Exports users to a comma-seperated-values file
    """
    cw = csv.writer(open(filename, "w"))
    cw.writerow(FIELD_NAMES)
    for name, u in users.iteritems():
        cw.writerow([u.name, u.uid, u.rname, u.office, u.wphone, u.hphone, 
            u.dir, u.shell, ",".join(u.groups), u.min, u.max, u.warn, u.inact, 
            u.expire, u.pwd, u.plainpw])


def import_users_from_csv(filename):
    """
    Imports users from a comma-seperated-values file
    """
    cr = csv.reader(open(filename, "r"))
    if cr.next() != FIELD_NAMES:
        print "Wrong headers in .csv file. Please use an exported file as a \
            template, and don't change the headers or move columns."
        return {}

    users={}
    for row in cr:
        u=User(*row)
        u.groups = u.groups.split(',')
        users[u.name]=u

    return users


def import_users_from_sch(clipboard):
    """
    Imports users from the register.sch.gr/studentsadmin web page.
    """
    header = "A.A\tΕνεργός\tΌνομα Χρήστη\tΕπώνυμο\tΌνομα\tΚηδεμόνας\tΑρ. \
        Μητρώου\tΤμήμα\tΤύπος Λογαριασμού"

    clipboard = re.sub(" *\t *", "\t", clipboard)
    second_line = clipboard.split("\n",3)[1:2]

    if second_line != [header]:
        print "Unexpected header line:", "\n".join(second_line)
        return {}
    usersdata = re.findall('([0-9 ]*)\t\t(.*)\t(.*)\t(.*)\t(.*)\t(.*)\t(.*)\t(.*)', 
        clipboard)

    users = {}
    for user in usersdata:
        u = User(name=user[1].strip(), rname=user[2].strip() + ' ' + 
                user[3].strip(), office=user[4].strip(), 
                groups=[user[6].strip(), user[7].strip()] + DEFAULT_GROUPS,
                plainpw=user[1].strip())
        users[u.name] = u

    return users
    

def mkpasswd(plainpw):
    """
    Converts a plain text password to a sha-512 encrypted one.
    """
    alphabet='0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'
    salt=''.join([ random.choice(alphabet) for i in range(8) ])

    return crypt.crypt(plainpw, "$6$%s$" % salt)

def add_group(gname, gid=""):
    """
    Adds a group to /etc/group.
    Returns "" on success or the output if addgroup failed.
    """
    # It's not an error if the group already exists
    grps = grp.getgrall()
    for g in grps:
        if g.gr_name == gname:
            return ""

    cmdline = ["addgroup", gname]
    if gid != "":
        cmdline.extend(["--gid", gid])

    return run_command(cmdline)


def add_user(u):
    """
    Adds a user to /etc/passwd.
    Returns "" on success or the output if any error occurred
    """
    cmdline = ["useradd", "--create-home", "--user-group"]

    if u.name == "":
        print "add_user: missing name"
        return False

    if u.uid != "":
        cmdline.extend(["--uid", u.uid])

    gecos = ','.join([u.rname, u.office, u.wphone, u.hphone])
    cmdline.extend(["--comment", gecos])

    if u.dir != "":
        cmdline.extend(["--home", u.dir])

    if u.shell != "":
        shell = u.shell
    else:
        shell = "/bin/bash"
    cmdline.extend(["--shell", shell])

    if u.min != "":
        cmdline.extend(["--key", "PASS_MIN_DAYS=%s" % u.min])

    if u.max != "":
        cmdline.extend(["--key", "PASS_MAX_DAYS=%s" % u.max])

    if u.warn != "":
        cmdline.extend(["--key", "PASS_WARN_AGE=%s" % u.warn])

    if u.inact != "":
        cmdline.extend(["--inactive", u.inact])

    if u.expire != "":
        cmdline.extend(["--expiredate", u.expire])

    # TODO: create the groups before using them
    groups = ','.join(u.groups)
    if groups == "":
        groups = ','.join(DEFAULT_GROUPS)
    cmdline.extend(["--groups", groups])

    if u.plainpw != "":
        pwd = mkpasswd(u.plainpw)
    else:
        pwd = u.pwd
    if pwd != "":
        cmdline.extend(["--password", pwd])

    cmdline.append(u.name)

    return run_command(cmdline)


def write_users_to_passwd(users, dirname="/etc"):
    for name, u in users.iteritems():
        add_user(u)


def print_bold(message):
    print ("\033[;;1m%s\033[;;m" % message)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

###########################################################################
# Global registry type stuff.
#
# Copyright (C) 2010 Fotis Tsamis <ftsamis@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FINESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# On Debian GNU/Linux systems, the complete text of the GNU General
# Public License can be found in `/usr/share/common-licenses/GPL".
###########################################################################

"""
Global registry type stuff
"""

knownClients = {}
timedOutClients = {}
knownGUIs = []


def clientConnected(handle, client):
#    print "Client connected: %s" % handle.encode("utf-8")
    knownClients[handle] = client
    for gui in knownGUIs:
        gui.clientConnected(handle)


def clientDisconnected(handle):
    if handle not in knownClients:
        print "Disconnect from unknown client: %s" % handle.encode("utf-8")
        return

#    print "Client disconnected: %s" % handle.encode("utf-8")
    del knownClients[handle]
    for gui in knownGUIs:
        gui.clientDisconnected(handle)

def clientTimedOut(handle):
    timedOutClients[handle] = knownClients[handle]
    clientDisconnected(handle)
    
def clientReconnected(handle):
    clientConnected(handle, timedOutClients[handle])
    del timedOutClients[handle]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###########################################################################
# UI connection.
#
# Copyright (C) 2010 Fotis Tsamis <ftsamis@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FINESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# On Debian GNU/Linux systems, the complete text of the GNU General
# Public License can be found in `/usr/share/common-licenses/GPL".
###########################################################################

import os

from twisted.internet import protocol
from twisted.protocols import amp

import commands

class Daemon(amp.AMP):

    def __init__(self, client):
        self.client = client


    def connectionMade(self):
        amp.AMP.connectionMade(self)
        self.client.connected(self)

    
    def connectionLost(self, reason):
        amp.AMP.connectionLost(self, reason)
        self.client.disconnected(self)
    
    @commands.ClientConnected.responder
    def clientConnected(self, handle):
        self.client.amp_clientConnected(handle)
        return {}


    @commands.ClientDisconnected.responder
    def clientDisconnected(self, handle):
        self.client.amp_clientDisconnected(handle)
        return {}


    def enumerateClients(self):
        d = self.callRemote(commands.EnumerateClients)
        d.addCallback(lambda r: r['handles'])
        return d
        

    def command(self, handle, command):
        d = self.callRemote(commands.ClientCommand,
                            handle=handle,
                            command=command)

        def gotResult(response):
            filename = response['filename']
            if filename:
                result = open(filename, 'rb').read()
                os.unlink(filename)
                return result
            else:
                return response['result']

        d.addCallback(gotResult)
        return d




#!/usr/bin/env python
# -*- coding: utf-8 -*-

###########################################################################
# Network benchmark.
#
# Copyright (C) 2016 Fotis Tsamis <ftsamis@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FINESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# On Debian GNU/Linux systems, the complete text of the GNU General
# Public License can be found in `/usr/share/common-licenses/GPL".
###########################################################################

import os
import subprocess
import fcntl
import gtk
import gtk.gdk as gdk
import pygtk
import gobject
from twisted.internet import reactor

from graph import Graph
from epoptes.common.constants import *


def humanize(value, decimal=1, unit=''):
    value = float(value)
    for prefix in ['', 'K', 'M', 'G', 'T', 'P', 'E', 'Z']:
        if abs(value) < 1000:
            value = round(value, decimal)
            return "%.*f %s%s" % (decimal, value, prefix, unit)
        value = value / 1000
    return "%.*f %s%s" % (decimal, value, 'Y', unit)


def bits_to_mbits(value):
    return float(value) / 1000 ** 2


def read_nonblocking(f):
        fd = f.fileno()
        fl = fcntl.fcntl(fd, fcntl.F_GETFL)
        fcntl.fcntl(fd, fcntl.F_SETFL, fl | os.O_NONBLOCK)
        try:
            return f.read()
        except:
            return None


class NetworkBenchmark:
    def __init__(self, parent, clients, execute):
        self.wTree = gtk.Builder()
        self.wTree.add_from_file('netbenchmark.ui')
        self.wTree.connect_signals(self)
        self.get = self.wTree.get_object
        self.parent = parent
        self.execute = execute
        self.clients_par = clients
        self.clients = {}
        self.iperf = None
        self.processes = {}
        self.results = {}
        self.dlg = self.get('benchmark_dialog')
        self.dlg.set_transient_for(self.parent)
        self.table = self.get('results_treeview')
        self.graph = None
        self.measurements_n = 0
        self.timeleft = 0
        self.timeout = 5
        self.output_timeout = None
        self.more_output = None


    def warning_message(self, msg):
        msgdlg = self.get('msgdlg')
        msgdlg.set_property("message-type", gtk.MESSAGE_WARNING)
        msgdlg.set_transient_for(self.dlg)
        msgdlg.set_title(_("Warning"))
        msgdlg.set_markup(msg)
        msgdlg.show_all()


    def error_message(self, msg):
        msgdlg = self.get('msgdlg')
        msgdlg.set_property("message-type", gtk.MESSAGE_ERROR)
        msgdlg.set_transient_for(self.dlg)
        msgdlg.set_title(_("Error"))
        msgdlg.set_markup(msg)
        msgdlg.show_all()


    def on_message_dialog_close(self, widget):
        self.get('msgdlg').hide()


    def store_pid(self, handle, pid):
        """Store the PID of the iperf process running on each client
        allowing us to kill it later.
        """
        self.processes[handle] = int(pid)


    def create_graph(self, entries):
        options = {
            'axis': {
                'x': {
                    'ticks': [dict(v=i, label=l[0]) for i, l in enumerate(entries)],
                    'rotate': 0,
                    'label' : 'Computers'
                },
                'y': {
                    #'tickCount': 15,
                    'tickPrecision' : 0,
                    #'range' : [20,1100],
                    #'interval' : 10,
                    'label' : 'MBits/s',
                    'rotate': 0
                }
            },
            'background': {
                'chartColor': '#FBFBFB',
                'baseColor': '#FBFBFB',
                'lineColor': '#444446'
            },
            'colorScheme': {
                'name': 'rainbow',
                'args': {
                    'initialColor': 'green',
                },
            },
            'legend': {
                'position': {
                    'right': 20,
                    'top' : 20
                    }
            },
            'padding': {
                'left': 2,
                'right' : 20,
                'top' : 2,
                'bottom': 2
            },
            'title': _('Epoptes Network Benchmark Results')
        }

        dataSet = (
            (_('Upload Rate'), [[i, l[1]] for i, l in enumerate(entries)]),
            (_('Download Rate'), [[i, l[2]] for i, l in enumerate(entries)])
        )

        g = Graph()
        g.set_options(options)
        g.set_data(dataSet)
        height = len(entries)*50+100
        g.set_size_request(-1, height)
        return g


    def run(self):
        if not self.clients_par:
            self.warning_message(_('There are no selected clients to run the benchmark on.'))
            return False

        # Check if there are offline clients or clients with no root client in the selection
        off = []
        for client in self.clients_par:
            inst = client[C_INSTANCE]
            if inst.hsystem:
                self.clients[inst.hsystem.split(':')[0]] = (inst.hsystem, inst.get_name())
            else:
                off.append(inst.get_name())

        if not self.clients:
            self.warning_message(_('All of the selected clients are either offline, or do not have epoptes-client running as root.'))
            return False
        # Order matters here
        self.dlg.show()
        if off:
            self.warning_message(_('The following clients will be excluded from the benchmark because they are either offline, or do not have epoptes-client running as root.') + '\n\n' + ', '.join(off))


    def start_benchmark(self, seconds):
        self.iperf = subprocess.Popen('iperf -s -xS -yC'.split(), 
                            stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        reactor.addSystemEventTrigger('before', 'shutdown', self.stop_benchmark)
        for client in self.clients:
            handle = self.clients[client][0]
            d = self.execute(handle, 'start_benchmark %d' % seconds)
            d.addCallback(lambda r, h=client : self.store_pid(h, r))


    def stop_benchmark(self):
        for client in self.clients:
            break
            if client in self.processes:
                handle = self.clients[client][0]
                pid = self.processes[client]
                self.execute(handle, 'stop_benchmark %d' % pid)
        if self.iperf and self.iperf.poll() is None:
            self.iperf.kill()


    def on_btn_startBenchmark_clicked(self, widget):
        seconds = int(self.get('seconds_adjustment').get_value())
        # Half time for upload speed and half for download
        self.start_benchmark(seconds/2)
        self.timeleft = seconds
        self.get('seconds_spinbox').set_sensitive(False)
        self.get('hbox_buttons').set_visible(False)
        self.get('time_left_label').set_text(_("Benchmark finishing in %d seconds...") % self.timeleft)
        self.countdown_event = gobject.timeout_add(1000, self.update_time_left)
        self.get('hbox_status').set_visible(True)


    def update_time_left(self):
        self.timeleft -= 1
        # Check if the server has exited for some reason
        if self.iperf.poll() is not None:
            message = read_nonblocking(self.iperf.stderr)
            self.error_message(_("Something went wrong with the iperf server process:\n\n%s") % message)
            self.cancel_benchmark()
            return False
        
        if self.timeleft == 0:
            self.get('cancel_btn').set_visible(False)
            self.get('time_left_label').set_text(_("Processing data..."))
            self.get_results()
            return False
        self.get('time_left_label').set_text(_("Benchmark finishing in %d seconds...") % self.timeleft)
        return True


    def parse_iperf_output(self, output):
        """Parse 'output' as a string of single or multiple lines of CSV in the form of
        timestamp,server_ip,port,client_ip,port,id,from-to,transfered(Bytes),bandwidth(bps)
        and populate a dict of client_ip : [upload Mbps, download Mbps] pairs
        storing it in self.results.
        """
        data = output.strip().split()
        for line in data:
            values = line.split(',')
            if len(values) != 9:
                continue
            client_ip = values[3]
            client_port = values[4] # will be 5001 if the client is receiving
            bandwidth = int(values[8])
            if client_ip in self.clients:
                if client_ip not in self.results:
                    self.results[client_ip] = [0, 0]

                if client_port == "5001":
                    # Download (bits/s)
                    self.results[client_ip][1] = int(bandwidth)
                    self.measurements_n += 1
                else:
                    # Upload (bits/s)
                    self.results[client_ip][0] = int(bandwidth)
                    self.measurements_n += 1


    def get_more_output(self):
        output = read_nonblocking(self.iperf.stdout)
        if output:
            self.parse_iperf_output(output)
        if self.measurements_n == len(self.clients)*2:
            self.show_results()
            return False
        return True


    def get_results(self):
        self.more_output = gobject.timeout_add(200, self.get_more_output)
        self.output_timeout = gobject.timeout_add(self.timeout*1000, self.show_results, True)
        

    def data_func(self, column, cell, model, iter, index):
        bps = model[iter][index]
        if bps <= 0:
            cell.set_property("text", "—")
        else:
            cell.set_property("text", humanize(bps, unit='bps'))


    def show_results(self, timed_out=False):
        # At this point we either have all our output or we give up waiting
        self.stop_benchmark()
        if self.output_timeout:
            gobject.source_remove(self.output_timeout)
        if self.more_output:
            gobject.source_remove(self.more_output)
        
        upload_col = self.get('upload_column')
        download_col = self.get('download_column')
        upload_col.set_cell_data_func(self.get('cellrenderertext2'), self.data_func, 1)
        download_col.set_cell_data_func(self.get('cellrenderertext3'), self.data_func, 2)
        
        results_n = len(self.results)
        if results_n > 0:
            graph_entries = []
            total_up = 0
            total_down = 0
            # List all the clients regardless of if we received measurements
            for client_ip in self.clients:
                client_name = self.clients[client_ip][1]
                if client_ip in self.results:
                    up, down = self.results[client_ip]
                else:
                    up, down = (0,0)
                graph_entries.append((client_name, bits_to_mbits(up), bits_to_mbits(down)))
                self.get('results_store').append([client_name, up, down])
                total_up += up
                total_down += down
            
            self.graph = self.create_graph(graph_entries)
            self.graph.set_visible(True)
            self.dlg.set_visible(False)
            results_dlg = self.get('results_dialog')
            results_dlg.set_transient_for(self.parent)
            results_dlg.show_all()
            
            clients_n = len(self.clients)
            self.get('avg_client').set_text(humanize(total_up / clients_n, unit='bps'))
            self.get('total_client').set_text(humanize(total_up, unit='bps'))
            self.get('avg_server').set_text(humanize(total_down / clients_n, unit='bps'))
            self.get('total_server').set_text(humanize(total_down, unit='bps'))
            if timed_out:
                self.get('warning_hbox').show()
            else:
                self.get('warning_hbox').hide()
        else:
            self.error_message(_("Did not get measurements from any of the clients. Check your network settings."))
            self.dlg.set_visible(False)


    def cancel_benchmark(self):
        self.stop_benchmark()
        gobject.source_remove(self.countdown_event)
        self.get('hbox_status').set_visible(False)
        self.get('seconds_spinbox').set_sensitive(True)
        self.get('hbox_buttons').set_visible(True)


    def on_cancel_btn_clicked(self, widget):
        self.cancel_benchmark()


    def on_close_button_clicked(self, widget):
        self.get('results_dialog').destroy()


    def show_graph_toggled(self, widget):
        viewport = self.get('viewport')
        if viewport.get_child() == self.table:
            if self.graph:
                viewport.remove(self.table)
                viewport.add(self.graph)
        else:
            viewport.remove(self.graph)
            viewport.add(self.table)


    def on_btn_close_clicked(self, widget):
        self.dlg.destroy()


    def on_window_destroy(self, widget, event):
        self.dlg.destroy()




#!/usr/bin/env python
# -*- coding: utf-8 -*-

###########################################################################
# Commands.
#
# Copyright (C) 2010 Fotis Tsamis <ftsamis@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FINESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# On Debian GNU/Linux systems, the complete text of the GNU General
# Public License can be found in `/usr/share/common-licenses/GPL".
###########################################################################

from twisted.protocols import amp


class ClientConnected(amp.Command):
    arguments = [('handle', amp.Unicode())]
    response = []


class ClientDisconnected(amp.Command):
    arguments = [('handle', amp.Unicode())]
    response = []


class EnumerateClients(amp.Command):
    arguments = []
    response = [('handles', amp.ListOf(amp.Unicode()))]


class ClientCommand(amp.Command):
    arguments = [('handle', amp.Unicode()),
                 ('command', amp.Unicode())]

    response = [('result', amp.String()),
                ('filename', amp.String())]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###########################################################################
# Classes for creating client and group objects.
#
# Copyright (C) 2011 Fotis Tsamis <ftsamis@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FINESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# On Debian GNU/Linux systems, the complete text of the GNU General
# Public License can be found in `/usr/share/common-licenses/GPL".
###########################################################################

clients = []
class Client:
    def __init__(self, type='', mac='', hostname='', alias='', users={}, hsystem=''):
        self.type = type
        self.mac = mac.upper()
        self.hostname = hostname
        self.alias = alias
        self.users = users.copy()
        self.hsystem = hsystem
        clients.append(self)
    
    def set_offline(self):
        self.hsystem = ''
        self.users = {}
        self.type = 'offline'
        self.hostname = ''
    
    def is_offline(self):
        """Return True only if there is no system or session handle
        for this client but there is a MAC address.
        """
        return not(self.users or self.hsystem) and self.mac
        
    def get_name(self):
        """Return the alias of the client or the hostname if the 
        alias is not set, or if both are unset (offline clients)
        return the MAC address.
        """
        return self.alias or self.hostname or self.mac
        
    def set_name(self, name):
        self.alias = name
        
    def add_user(self, username, realname, handle):
        self.users[handle] = {'uname' : username, 'rname' : realname}


class Group:
    def __init__(self, name="New group", members={}):
        self.name = name
        #{<instance> : {'x_pos':342, 'y_pos':112, 'size':600}}
        self.members = members.copy()
    
    def get_members(self):
        """Return a list with all the clients that are members of the group"""
        return self.members.keys()
    
    def has_client(self, client):
        """Check if a client is member of the group"""
        return client in self.members
    
    def add_client(self, client, **props):
        """Add a client to the group"""
        self.members[client] = {}
        self.set_properties(client, **props)
    
    def remove_client(self, client):
        """Remove a client from the group"""
        del self.members[client]
    
    def set_properties(self, client, **props):
        """Set x_pos, y_pos or size property for a client on the group"""
        for prop, value in props.iteritems():
            if isinstance(value, (int, float)):
                self.members[client][prop] = value
                #TODO: Save the new values to the disk.
            else:
                raise TypeError("The value of '%s' must be a number." % prop)
                
    def get_property(self, client, prop):
        """Return the value of the 'prop' property for client"""
        return self.members[client][prop]
    



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###########################################################################
# Graph generator.
#
# Copyright (C) 2016 Fotis Tsamis <ftsamis@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FINESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# On Debian GNU/Linux systems, the complete text of the GNU General
# Public License can be found in `/usr/share/common-licenses/GPL".
###########################################################################

import gtk
import cairo
from pycha.bar import HorizontalBarChart

class Graph(gtk.DrawingArea):
    def __init__(self):
        gtk.DrawingArea.__init__(self)
        self.connect("expose-event", self.expose)
        self.connect("size-allocate", self.size_allocate)
        self._surface = None
        self._options = None
 
    def set_options(self, options):
        self._options = options
 
    def set_data(self, data):
        self._data = data
        self.queue_draw()
 
    def plot(self):
        chart = HorizontalBarChart(self._surface, self._options)
        chart.addDataset(self._data)
        chart.render()
 
    def expose(self, widget, event):
        context = widget.window.cairo_create()
        context.rectangle(event.area.x, event.area.y,
                          event.area.width, event.area.height)
        context.clip()
        self.draw(context)
        return False
 
    def draw(self, context):
        rect = self.get_allocation()
        self._surface = cairo.ImageSurface(cairo.FORMAT_ARGB32,
                                           rect.width, rect.height)
        self.plot()
        context.set_source_surface(self._surface, 0, 0)
        context.paint()
 
    def size_allocate(self, widget, requisition):
        self.queue_draw()




#!/usr/bin/env python
# -*- coding: utf-8 -*-

###########################################################################
# GUI.
#
# Copyright (C) 2010 Fotis Tsamis <ftsamis@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FINESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# On Debian GNU/Linux systems, the complete text of the GNU General
# Public License can be found in `/usr/share/common-licenses/GPL".
###########################################################################

import pygtk
pygtk.require('2.0')

import gtk
import gobject
import os
import re
import dbus
import logging
import sys
import shlex
import pipes
import getpass
import socket
import locale
from gobject import TYPE_PYOBJECT as gPyobject
from twisted.internet import reactor
from twisted.python import log

from epoptes import __version__
from epoptes import ui
from notifications import *
from execcommand import *
from sendmessage import *
from about_dialog import About
from client_information import ClientInformation
from benchmark import NetworkBenchmark
from epoptes.daemon import uiconnection
from epoptes.core.lib_users import *
from epoptes.common import ltsconf
from epoptes.common import config
from epoptes.common.constants import *
from epoptes.core import wol
from epoptes.core import structs


class EpoptesGui(object):
    
    def __init__(self):
        self.shownCompatibilityWarning = False
        self.vncserver = None
        self.vncviewer = None
        self.scrWidth = 100
        self.scrHeight = 75
        self.showRealNames = False
        self.currentScreenshots = dict()
        self.current_macs = subprocess.Popen(['sh', '-c',
            """ip -oneline -family inet link show | sed -n '/.*ether[[:space:]]*\\([[:xdigit:]:]*\).*/{s//\\1/;y/abcdef-/ABCDEF:/;p;}'
            echo $LTSP_CLIENT_MAC"""],
            stdout=subprocess.PIPE).communicate()[0].split()
        self.uid = os.getuid()
        if 'thumbnails_width' in config.user:
            self.scrWidth = config.user['thumbnails_width']
        self.offline = gtk.gdk.pixbuf_new_from_file('images/offline.svg')
        self.thin = gtk.gdk.pixbuf_new_from_file('images/thin.svg')
        self.fat = gtk.gdk.pixbuf_new_from_file('images/fat.svg')
        self.standalone = gtk.gdk.pixbuf_new_from_file('images/standalone.svg')
        self.imagetypes = {'thin' : self.thin, 'fat' : self.fat,
            'standalone' : self.standalone, 'offline' : self.offline}
        
        self.wTree = gtk.Builder()
        self.wTree.add_from_file('epoptes.ui')
        
        self.get = lambda obj: self.wTree.get_object(obj)
        
        # Connect glade handlers with the callback functions
        self.wTree.connect_signals(self)
        
        
        # Hide the remote assistance menuitem if epoptes-client is not installed
        if not os.path.isfile('/usr/share/epoptes-client/remote-assistance'):
            self.get('mi_remote_assistance').set_property('visible', False)
            self.get('remote_assistance_separator').set_property('visible', False)
        
        self.groups_menu = self.get('mAddToGroup')
        self.add_to_group_menu = self.get('miAddToGroup')
        
        self.gstore = gtk.ListStore(str, object, bool)
        
        self.gtree = self.get("groups_tree")
        self.gtree.set_model(self.gstore)
        self.gtree.get_selection().connect("changed", self.on_group_selection_changed)
        
        self.mainwin = self.get('mainwindow')
        
        self.cstore = gtk.ListStore(str, gtk.gdk.Pixbuf, object, str)
        self.cview = self.get('clientsview')
        self.cView_order = (1, 0)
        self.set_cView(*self.cView_order)
        
        self.cview.set_model(self.cstore)
        self.cview.set_pixbuf_column(C_PIXBUF)
        self.cview.set_text_column(C_LABEL)
        
        self.cstore.set_sort_column_id(C_LABEL, gtk.SORT_ASCENDING)
        self.on_clients_selection_changed()
        
        self.cview.enable_model_drag_source(gtk.gdk.BUTTON1_MASK, [("add", gtk.TARGET_SAME_APP, 0)], gtk.gdk.ACTION_COPY)
        self.gtree.enable_model_drag_dest([("add", gtk.TARGET_SAME_APP, 0)], gtk.gdk.ACTION_COPY)
        
        self.default_group = structs.Group('<b>'+_('Detected clients')+'</b>')
        default_iter = self.gstore.append([self.default_group.name, self.default_group, False])
        self.default_group.ref = gtk.TreeRowReference(self.gstore, self.gstore.get_path(default_iter))
        self.gtree.get_selection().select_path(self.default_group.ref.get_path())
        
        self.get('iconsSizeAdjustment').set_value(self.scrWidth)
        self.reload_imagetypes()
        
        saved_clients, groups = config.read_groups(os.path.expanduser('~/.config/epoptes/groups.json'))
        if len(groups) > 0:
            self.add_to_group_menu.set_sensitive(True)
        for grp in groups:
            self.gstore.append([grp.name, grp, True])
            mitem = gtk.MenuItem(grp.name)
            mitem.show()
            mitem.connect('activate', self.on_add_to_group_clicked, grp)
            self.groups_menu.append(mitem)
        
        self.fillIconView(self.getSelectedGroup()[1])
        if config.settings.has_option('GUI', 'selected_group'):
            path = config.settings.getint('GUI', 'selected_group')
            self.gtree.get_selection().select_path(path)
        if config.settings.has_option('GUI', 'label'):
            try:
                self.get(config.settings.get('GUI', 'label')).set_active(True)
            except:
                pass
        if config.settings.has_option('GUI', 'showRealNames'):
            self.get('mcShowRealNames').set_active(config.settings.getboolean('GUI', 'showRealNames'))
        self.mainwin.set_sensitive(False)
        

    #################################################################
    #                       Callback functions                      #
    #################################################################
    def on_gtree_drag_motion(self, widget, context, x, y, etime):
        drag_info = widget.get_dest_row_at_pos(x, y)
        # Don't allow dropping in the empty space of the treeview,
        # or inside the 'Detected' group, or inside the currently selected group
        selected_path = self.gstore.get_path(self.getSelectedGroup()[0])
        if (not drag_info or drag_info[0] == self.default_group.ref.get_path() or
            drag_info[0] == selected_path):
            widget.set_drag_dest_row(None, gtk.TREE_VIEW_DROP_AFTER)
        else:
            path, pos = drag_info
            # Don't allow dropping between the groups treeview rows
            if pos == gtk.TREE_VIEW_DROP_BEFORE:
                widget.set_drag_dest_row(path, gtk.TREE_VIEW_DROP_INTO_OR_BEFORE)
            elif pos == gtk.TREE_VIEW_DROP_AFTER:
                widget.set_drag_dest_row(path, gtk.TREE_VIEW_DROP_INTO_OR_AFTER)
        
        context.drag_status(context.suggested_action, etime)
        return True
    
    def on_gtree_drag_drop(self, wid, context, x, y, time):
        dest = self.gtree.get_dest_row_at_pos(x, y)
        if dest is not None:
            path, pos = dest
            group = self.gstore[path][G_INSTANCE]
            if not group is self.default_group:
                for cln in self.getSelectedClients():
                    cln = cln[C_INSTANCE]
                    if not group.has_client(cln):
                        group.add_client(cln)
    
        context.finish(True, False, time)
        return True
    
    def save_settings(self):
        sel_group = self.gstore.get_path(self.getSelectedGroup()[0])[0]
        self.gstore.remove(self.gstore.get_iter(self.default_group.ref.get_path()))
        config.save_groups(os.path.expanduser('~/.config/epoptes/groups.json'), self.gstore)
        settings = config.settings
        if not settings.has_section('GUI'):
            settings.add_section('GUI')        
        
        settings.set('GUI', 'selected_group', sel_group)
        settings.set('GUI', 'showRealNames', self.showRealNames)
        settings.set('GUI', 'thumbnails_width', self.scrWidth)
        try:
            f = open(os.path.expanduser('~/.config/epoptes/settings'), 'w')
            settings.write(f)
            f.close()
        except:
            pass
    
    def on_mainwin_destroy(self, widget):
        """
        Quit clicked

        Close main window
        """
        self.save_settings()
        if not self.vncserver is None:
            self.vncserver.kill()
        if not self.vncviewer is None:
            self.vncviewer.kill()
        reactor.stop()

    def on_miBenchmark_activate(self, widget):
        dlg = NetworkBenchmark(self.mainwin, self.getSelectedClients() or self.cstore, self.daemon.command)
        dlg.run()

    def toggleRealNames(self, widget=None):
        """Show/hide the real names of the users instead of the usernames"""
        self.showRealNames = widget.get_active()
        for row in self.cstore:            
            self.setLabel(row)

    def wake_on_lan(self, widget):
        """Boot the selected computers with WOL"""
        clients = self.getSelectedClients()
        if len(clients) == 0: # No client selected, send the command to all
            clients = self.cstore
        for client in clients:
            # Make sure that only offline computers will be sent to wol
            client = client[C_INSTANCE]
            if client.is_offline():
                wol.wake_on_lan(client.mac)

    def poweroff(self, widget):
        """Shut down the selected clients."""
        self.execOnSelectedClients(["shutdown"],
            warn=_('Are you sure you want to shutdown all the computers?'))

    def reboot(self, widget):
        """Reboot the selected clients."""
        self.execOnSelectedClients(["reboot"],
            warn=_('Are you sure you want to reboot all the computers?'))


    def logout(self, widget):
        """Log off the users of the selected clients."""
        self.execOnSelectedClients(['logout'], mode=EM_SESSION,
            warn=_('Are you sure you want to log off all the users?'))


    def reverseConnection(self, widget, path, view_column, cmd, *args):
        # Open vncviewer in listen mode
        if self.vncviewer is None:
            self.vncviewerport = self.findUnusedPort()
            # If the user installed ssvnc, prefer it over xvnc4viewer
            if os.path.isfile('/usr/bin/ssvncviewer'):
                self.vncviewer = subprocess.Popen(['ssvncviewer',
                    '-multilisten', str(self.vncviewerport-5500)])
            elif os.path.isfile('/usr/bin/xvnc4viewer'):
                self.vncviewer = subprocess.Popen(['xvnc4viewer',
                    '-listen', str(self.vncviewerport)])
            # Support tigervnc on rpm distributions (LP: #1501747)
            elif os.path.isfile('/usr/share/locale/de/LC_MESSAGES/tigervnc.mo'):
                self.vncviewer = subprocess.Popen(['vncviewer',
                    '-listen', str(self.vncviewerport)])
            # The rest of the viewers, like tightvnc
            else:
                self.vncviewer = subprocess.Popen(['vncviewer',
                    '-listen', str(self.vncviewerport-5500)])

        # And, tell the clients to connect to the server
        self.execOnSelectedClients([cmd, self.vncviewerport] + list(args))


    def assistUser(self, widget, path=None, view_column=None):
        if config.settings.has_option('GUI', 'grabkbdptr'):
            grab = config.settings.getboolean('GUI', 'grabkbdptr')
        if grab:
            self.reverseConnection(widget, path, view_column, 'get_assisted', 'True')
        else:
            self.reverseConnection(widget, path, view_column, 'get_assisted')


    def monitorUser(self, widget, path=None, view_column=None):
        self.reverseConnection(widget, path, view_column, 'get_monitored')


    def findUnusedPort(self):
        """Find an unused port."""
        import socket

        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.bind(('', 0))
        s.listen(1)
        port = s.getsockname()[1]
        s.close()
        return port


    def _broadcastScreen(self, fullscreen=''):
        if self.vncserver is None:
            import random
            import string
            pwdfile=os.path.expanduser('~/.config/epoptes/vncpasswd')
            pwd=''.join(random.sample(string.letters + string.digits, 8))
            subprocess.call(['x11vnc', '-storepasswd', pwd, pwdfile])
            f=open(pwdfile)
            pwd=f.read()
            f.close()
            self.pwd=''.join('\\%o' % ord(c) for c in pwd)
            self.vncserverport = self.findUnusedPort()
            self.vncserver = subprocess.Popen(['x11vnc', '-noshm', '-nopw',
                '-quiet', '-viewonly', '-shared', '-forever', '-nolookup',
                '-24to32', '-threads', '-rfbport', str(self.vncserverport),
                '-rfbauth', pwdfile])
        # Running `xdg-screensaver reset` as root doesn't reset the D.E.
        # screensaver, so send the reset command to both epoptes processes
        self.execOnSelectedClients(['reset_screensaver'],
            mode=EM_SYSTEM_AND_SESSION)
        self.execOnSelectedClients(["receive_broadcast", self.vncserverport, 
            self.pwd, fullscreen], mode=EM_SYSTEM_OR_SESSION)


    def broadcastScreen(self, widget):
        self._broadcastScreen('true')


    def broadcastScreenWindowed(self, widget):
        self._broadcastScreen('')


    def stopTransmissions(self, widget):
        self.execOnClients(['stop_receptions'], self.cstore,
            mode=EM_SYSTEM_AND_SESSION)
        if not self.vncserver is None:
            self.vncserver.kill()
            self.vncserver = None


    ## FIXME FIXME: Should we allow for running arbitrary commands in clients?
    def execDialog(self, widget):
        cmd = startExecuteCmdDlg()
        # If Cancel or Close were clicked
        if cmd == '':
            return
        em = EM_SESSION
        if cmd[:5] == 'sudo ':
            em = EM_SYSTEM
            cmd = cmd[4:]
        self.execOnSelectedClients(['execute', cmd], mode=em)


    def sendMessageDialog(self, widget):
        msg = startSendMessageDlg()
        if msg:
            cmd = list(msg)
            cmd.insert(0, 'message')
            self.execOnSelectedClients(cmd)


    ## FIXME / FIXUS: Should we allow it?
    def openTerminal(self, em):
        clients = self.getSelectedClients()
        
        # If there is no client selected, send the command to all
        if len(clients) == 0:
            clients = self.cstore

        for client in clients:
            inst = client[C_INSTANCE]
            if inst.type == 'offline':
                continue

            port = self.findUnusedPort()
            
            user = '--'
            if em == EM_SESSION and client[C_SESSION_HANDLE]:
                user = inst.users[client[C_SESSION_HANDLE]]['uname']
            elif em == EM_SYSTEM:
                user = 'root'
            title = '%s@%s' % (user, inst.get_name())
            subprocess.Popen(['xterm', '-T', title, '-e', 'socat',
                'tcp-listen:%d,keepalive=1' % port, 'stdio,raw,echo=0'])
            self.execOnClients(['remote_term', port], [client], mode=em)


    def openUserTerminal(self, widget):
        self.openTerminal(EM_SESSION)


    def openRootTerminal(self, widget):
        self.openTerminal(EM_SYSTEM)


    def remoteRootTerminal(self, widget):
        self.execOnSelectedClients(['root_term'], mode=EM_SYSTEM)
    ## END_FIXUS


    def lockScreen(self, widget):
        """
        Lock screen for all the selected clients, displaying a message
        """
        msg = _("The screen is locked by a system administrator.")
        self.execOnSelectedClients(['lock_screen', 0, msg])


    def unlockScreen(self, widget):
        """
        Unlock screen for all clients selected
        """
        self.execOnSelectedClients(['unlock_screen'], mode=EM_SESSION_AND_SYSTEM)


    def soundOff(self, widget):
        """
        Disable sound usage for clients selected
        """
        self.execOnSelectedClients(['mute_sound', 0], mode=EM_SYSTEM_OR_SESSION)


    def soundOn(self, widget):
        """
        Enable sound usage for clients selected
        """
        self.execOnSelectedClients(['unmute_sound'], mode=EM_SYSTEM_AND_SESSION)


    def on_add_to_group_clicked(self, widget, group):
        clients = self.getSelectedClients()
        for client in clients:
            if not group.has_client(client[C_INSTANCE]):
                group.add_client(client[C_INSTANCE])


    def on_remove_from_group_clicked(self, widget):
        clients = self.getSelectedClients()
        group = self.getSelectedGroup()[1]
        
        if self.warnDlgPopup(_('Are you sure you want to remove the selected client(s) from group "%s"?') %group.name):
            for client in clients:
                group.remove_client(client[C_INSTANCE])
            self.fillIconView(self.getSelectedGroup()[1], True)


    def set_move_group_sensitivity(self):
        selected = self.getSelectedGroup()
        selected_path = self.gstore.get_path(selected[0])[0]
        blocker = not selected[1] is self.default_group
        self.get('move_group_up').set_sensitive(blocker and selected_path > 1)
        self.get('move_group_down').set_sensitive(blocker and selected_path < len(self.gstore)-1)


    def on_move_group_down_clicked(self, widget):
        selected_group_iter = self.getSelectedGroup()[0]
        path = self.gstore.get_path(selected_group_iter)[0]
        self.gstore.swap(selected_group_iter, self.gstore.iter_next(selected_group_iter))
        self.set_move_group_sensitivity()
        mitem = self.groups_menu.get_children()[path-1]
        self.groups_menu.reorder_child(mitem, path)


    def on_move_group_up_clicked(self, widget):
        selected_group_iter = self.getSelectedGroup()[0]
        path = self.gstore.get_path(selected_group_iter)[0]
        previous_iter = self.gstore.get_iter(path-1)
        self.gstore.swap(selected_group_iter, previous_iter)
        self.set_move_group_sensitivity()
        mitem = self.groups_menu.get_children()[path-1]
        self.groups_menu.reorder_child(mitem, path-2)
        


    def on_remove_group_clicked(self, widget):
        group_iter = self.getSelectedGroup()[0]
        group = self.gstore[group_iter][G_INSTANCE]
        
        if self.warnDlgPopup(_('Are you sure you want to remove group "%s"?') % group.name):
            path = self.gstore.get_path(group_iter)[0]
            self.gstore.remove(group_iter)
            menuitem = self.groups_menu.get_children()[path-1]
            self.groups_menu.remove(menuitem)


    def on_add_group_clicked(self, widget):
        new_group = structs.Group()
        iter = self.gstore.append([new_group.name, new_group, True])
        # Edit the name of the newly created group
        self.gtree.set_cursor(self.gstore.get_path(iter), self.get('group_name_column'), True)
        self.appendToGroupsMenu(new_group)


    def on_group_renamed(self, widget, path, new_name):
        self.gstore[path][G_LABEL] = new_name
        self.gstore[path][G_INSTANCE].name = new_name
        self.groups_menu.get_children()[int(path)-1].set_label(new_name)


    #FIXME: Remove the second parameter, find a better way
    def on_tb_client_properties_clicked(self, widget=None):
        dlg = ClientInformation(self.getSelectedClients(), self.daemon.command)
        if self.isDefaultGroupSelected():
            dlg.edit_button.set_sensitive(False)
        dlg.run()
        self.setLabel(self.getSelectedClients()[0])


    def on_mi_remote_assistance_activate(self, widget=None):
        path = '/usr/share/epoptes-client'
        subprocess.Popen('%s/remote-assistance' %path, shell=True, cwd=path)


    def on_mi_about_activate(self, widget=None):
        About().run()


    def on_cViewHU_toggled(self, mitem):
        self.set_cView(1, 0)
        config.settings.set('GUI', 'label', 'miClientsViewHostUser')


    def on_cViewUH_toggled(self, mitem):
        self.set_cView(0, 1)
        config.settings.set('GUI', 'label', 'miClientsViewUserHost')


    def on_cViewH_toggled(self, mitem):
        self.set_cView(-1, 0)
        config.settings.set('GUI', 'label', 'miClientsViewHost')


    def on_cViewU_toggled(self, mitem):
        self.set_cView(0, -1)
        config.settings.set('GUI', 'label', 'miClientsViewUser')


    def set_cView(self, user_pos= -1, name_pos=0):
        # Save the order so all new clients get the selected format
        self.cView_order = (user_pos, name_pos)
        for row in self.cstore:            
            self.setLabel(row)


    def connected(self, daemon):
        self.mainwin.set_sensitive(True)
        self.daemon = daemon
        daemon.enumerateClients().addCallback(lambda h: self.amp_gotClients(h))
        self.fillIconView(self.getSelectedGroup()[1])

    def disconnected(self, daemon):
        self.mainwin.set_sensitive(False)
        # If the reactor is not running at this point it means that we were
        # closed normally.
        if not reactor.running:
            return
        self.save_settings()
        msg = _("Lost connection with the epoptes service.")
        msg += "\n\n" + _("Make sure the service is running and then restart epoptes.")
        dlg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK,
                                 message_format=msg)
        dlg.set_title(_('Service connection error'))
        dlg.run()
        dlg.destroy()
        reactor.stop()
        

    # AMP callbacks
    def amp_clientConnected(self, handle):
        print "New connection from", handle
        d = self.daemon.command(handle, u'info')
        d.addCallback(lambda r: self.addClient(handle, r))
        d.addErrback(lambda err: self.printErrors("when connecting client %s: %s" %(handle, err)))


    def amp_clientDisconnected(self, handle):
        print "Disconnect from", handle
        
        def determine_offline(client):
            if client.hsystem == '' and client.users == {}:
                client.set_offline()
        client = None
        for client in structs.clients:
            if client.hsystem == handle:
                if self.getSelectedGroup()[1].has_client(client) or self.isDefaultGroupSelected():
                    shutdownNotify(client.get_name())
                client.hsystem = ''
                determine_offline(client)
                break
            
            elif handle in client.users:
                if self.getSelectedGroup()[1].has_client(client) or self.isDefaultGroupSelected():
                    logoutNotify(client.users[handle]['uname'], client.get_name())
                del client.users[handle]
                determine_offline(client)
                break
            else:
                client = None

        if not client is None:
            for row in self.cstore:
                if row[C_INSTANCE] is client: 
                    self.fillIconView(self.getSelectedGroup()[1], True)
                    break


    def amp_gotClients(self, handles):
        print "Got clients:", ', '.join(handles) or 'None'
        for handle in handles:
            d = self.daemon.command(handle, u'info')
            d.addCallback(lambda r, h=handle: self.addClient(h, r, True))
            d.addErrback(lambda err: self.printErrors("when enumerating client %s: %s" %(handle, err)))


    def on_button_close_clicked(self, widget):
        self.get('warningDialog').hide()


    def on_group_selection_changed(self, treeselection):
        self.cstore.clear()
        selected = self.getSelectedGroup()
                
        if selected is not None:
            self.fillIconView(selected[1])
            path = self.gstore.get_path(selected[0])[0]
            self.groups_menu.foreach(lambda w : w.set_sensitive(True))
            menuitems = self.groups_menu.get_children()
            if path != 0 and path-1 < len(menuitems):
                menuitems[path-1].set_sensitive(False)
        else:
            if not self.default_group.ref.valid():
                return
            self.gtree.get_selection().select_path(self.default_group.ref.get_path())
        self.get('remove_group').set_sensitive(not self.isDefaultGroupSelected())
        self.set_move_group_sensitivity()


    def addToIconView(self, client):
        """Properly add a Client class instance to the clients iconview."""
        # If there are one or more users on client, add a new iconview entry
        # for each one of them.
        
        label = 'uname'
        if self.showRealNames:
                label = 'rname'
        if client.users:
            for hsession, user in client.users.iteritems():
                self.cstore.append([self.calculateLabel(client, user[label]), self.imagetypes[client.type], client, hsession])
                self.askScreenshot(hsession, True)
        else:
            self.cstore.append([self.calculateLabel(client), self.imagetypes[client.type], client, ''])


    def fillIconView(self, group, keep_selection=False):
        """Fill the clients iconview from a Group class instance."""
        if keep_selection:
            selection = [row[C_INSTANCE] for row in self.getSelectedClients()]
        self.cstore.clear()
        if self.isDefaultGroupSelected():
            clients_list = [client for client in structs.clients if client.type != 'offline']
        else:
            clients_list = group.get_members()
        # Add the new clients to the iconview
        for client in clients_list:
            self.addToIconView(client)
        if keep_selection:
            for row in self.cstore:
                if row[C_INSTANCE] in selection:
                    self.cview.select_path(row.path)
                    selection.remove(row[C_INSTANCE])


    def isDefaultGroupSelected(self):
        """Return True if the default group is selected"""
        if self.getSelectedGroup():
            return self.getSelectedGroup()[1] is self.default_group
        return True


    def getSelectedGroup(self):
        """Return a 2-tuple containing the iter and the instance
        for the currently selected group."""
        iter = self.gtree.get_selection().get_selected()[1]
        if iter:
            return (iter, self.gstore[iter][G_INSTANCE])
        else:
            return None


    def addClient(self, handle, r, already=False):
        # already is True if the client was started before epoptes
        print "---\n**addClient's been called for", handle
        try:
            info = {}
            for line in r.strip().split('\n'):
                key, value = line.split('=', 1)
                info[key.strip()] = value.strip()
            user, host, ip, mac, type, uid, version, name = \
                info['user'], info['hostname'], info['ip'], info['mac'], \
                info['type'], int(info['uid']), info['version'], info['name']
        except:
            print "Can't extract client information, won't add this client"
            return
        
        # Check if the incoming client is the same with the computer in which
        # epoptes is running, so we don't add it to the list.
        if (mac in self.current_macs) and ((uid == self.uid) or (uid == 0)):
            print "* Won't add this client to my lists"
            return False
        
        print '  Continuing inside addClient...'
        
        # Compatibility check
        if [int(x) for x in re.split('[^0-9]*', version)] < COMPATIBILITY_VERSION:
            if not self.shownCompatibilityWarning:
                self.shownCompatibilityWarning = True
                dlg = self.get('warningDialog')
                # Show different messages for LTSP clients and standalones.
                msg = _("A connection attempt was made by a client with version %s, \
which is incompatible with the current epoptes version.\
\n\nYou need to update your clients to the latest epoptes-client version.") % version
                dlg.set_property('text', msg)
                dlg.set_transient_for(self.mainwin)
                dlg.show()
            self.daemon.command(handle, u"exit")
            return False
        sel_group = self.getSelectedGroup()[1]
        client = None
        for inst in structs.clients:
            # Find if the new handle is a known client
            if mac == inst.mac:
                client = inst
                print '* This is an existing client'
                break
        if client is None:
            print '* This client is a new one, creating an instance'
            client = structs.Client(mac=mac)
            
        # Update/fill the client information
        client.type, client.hostname = type, host
        if uid == 0:
            # This is a root epoptes-client
            print '* I am a root client'
            client.hsystem = handle
        else:
            # This is a user epoptes-client
            print '* I am a user client, will add', user, 'in my list'
            client.add_user(user, name, handle)
            if not already and (sel_group.has_client(client) or self.isDefaultGroupSelected()):
                loginNotify(user, host)
        
        if sel_group.has_client(client) or self.isDefaultGroupSelected():
            self.fillIconView(sel_group, True)


    def setLabel(self, row):
        inst = row[C_INSTANCE]
        if row[C_SESSION_HANDLE]:            
            label = 'uname'
            if self.showRealNames:
                label = 'rname'
            user = row[C_INSTANCE].users[row[C_SESSION_HANDLE]][label]
        else:
            user = ''
        row[C_LABEL] = self.calculateLabel(inst, user)


    def calculateLabel(self, client, username=''):
        """Return the iconview label from a hostname/alias
        and a username, according to the user options.
        """
        user_pos, name_pos = self.cView_order
        
        alias = client.get_name()
        if username == '' or user_pos == -1:
            return alias
        else:
            if user_pos == 0:
                label = username
                if name_pos == 1:
                    label += " (%s)" % alias
            elif name_pos == 0:
                label = alias
                if user_pos == 1:
                    label += " (%s)" % username
            return label


    def getShowRealNames(self):
        return self.get('')
    

    def askScreenshot(self, handle, firstTime=False):
        # Should always return False to prevent glib from calling us again
        if firstTime:
            if not handle in self.currentScreenshots:
                # Mark that we started asking for screenshots, but didn't yet get one
                self.currentScreenshots[handle] = None
            else:
                # We're already asking the client for screenshots, reuse the existing one
                if not self.currentScreenshots[handle] is None:
                    for i in self.cstore:
                        if handle == i[C_SESSION_HANDLE]:
                            self.cstore[i.path][C_PIXBUF] = self.currentScreenshots[handle]
                            break
                return False
        # TODO: Implement this using gtk.TreeRowReferences instead
        # of searching the whole model (Need to modify execOnClients)
        for client in self.cstore:
            if handle == client[C_SESSION_HANDLE]:
                self.execOnClients(['screenshot', self.scrWidth, self.scrHeight], handles=[handle], 
                                   reply=self.gotScreenshot)
                return False
        # That handle is no longer in the cstore, remove it
        try: del self.currentScreenshots[handle]
        except: pass
        return False


    def gotScreenshot(self, handle, reply):
        for i in self.cstore:
            if handle == i[C_SESSION_HANDLE]:
                # We want to ask for thumbnails every 5 sec after the last one.
                # So if the client is too stressed and needs 7 secs to
                # send a thumbnail, we'll ask for one every 12 secs.
                gobject.timeout_add(5000, self.askScreenshot, handle)
#                print "I got a screenshot from %s." % handle
                if not reply:
                    return
                try:
                    rowstride, size, pixels = reply.split('\n', 2)
                except:
                    return
                rowstride = int(rowstride)
                width, height = size.split('x')
                pxb = gtk.gdk.pixbuf_new_from_data(pixels,
                    gtk.gdk.COLORSPACE_RGB, False, 8, int(width), int(height),
                    rowstride)
                self.currentScreenshots[handle] = pxb
                self.cstore[i.path][C_PIXBUF] = pxb
                return
        # That handle is no longer in the cstore, remove it
        try: del self.currentScreenshots[handle]
        except: pass


    def getSelectedClients(self):
        selected = self.cview.get_selected_items()
        items = []
        for i in selected:
            items.append(self.cstore[i])
        return items

    def appendToGroupsMenu(self, group):
        mitem = gtk.MenuItem(group.name)
        mitem.show()
        mitem.connect('activate', self.on_add_to_group_clicked, group)
        self.groups_menu.append(mitem)
    
    def removeFromGroupsMenu(self, group):
        mitem = gtk.MenuItem(group.name)
        mitem.show()
        mitem.connect('activate', self.on_add_to_group_clicked, group)
        self.groups_menu.append(mitem)

    def changeHostname(self, mac, new_name):
        pass #FIXME: Implement this (virtual hostname)


    def openLink(self, link):
        subprocess.Popen(["xdg-open", link])


    def openHelpLink(self, widget):
        self.openLink("http://www.epoptes.org")


    def openBugsLink(self, widget):
        self.openLink("https://bugs.launchpad.net/epoptes")


    def openQuestionsLink(self, widget):
        self.openLink("https://answers.launchpad.net/epoptes")


    def openTranslationsLink(self, widget):
        self.openLink("http://www.epoptes.org/translations")


    def openIRCLink(self, widget):
        host = socket.gethostname()
        user = getpass.getuser()
        lang = locale.getlocale()[0]
        self.openLink("http://ts.sch.gr/repo/irc?user=%s&host=%s&lang=%s" % \
                       (user, host, lang))
    
    
    def iconsSizeScale_button_event(self, widget, event):
        """A hack to make the left click work like middle click
        in the scale (for jump-to functionality). Right click resets
        the thumbnail size.
        """
        if event.button == 1:
            event.button = 2
        elif event.button == 3:
            self.iconsSizeScaleChanged(None, 120) # Reset the thumbnail size
        return False
    
    
    def reload_imagetypes(self):
        """Improve the quality of previously resized svg icons,
        by reloading them.
        """
        old_pixbufs = self.imagetypes.values()
        loadSVG = lambda path: gtk.gdk.pixbuf_new_from_file_at_size(path, 
                                                  self.scrWidth, self.scrHeight)
        self.imagetypes = {
            'offline' : loadSVG('images/offline.svg'),
            'thin' : loadSVG('images/thin.svg'),
            'fat' : loadSVG('images/fat.svg'),
            'standalone' : loadSVG('images/standalone.svg')
        }
        
        rows = [row for row in self.cstore if row[C_PIXBUF] in old_pixbufs]
        for row in rows:
            row[C_PIXBUF] = self.imagetypes[row[C_INSTANCE].type]
    
    
    def on_iconsSizeScale_button_release_event(self, widget=None, event=None):
        # Here we want to resize the SVG icons from imagetypes at a better
        # quality than this of the quick pixbuf scale, since we assume that
        # the user has decided the desired zoom level.
        self.reload_imagetypes()
    
    
    def iconsSizeScaleChanged(self, widget=None, width=None):
        adj = self.get('iconsSizeAdjustment')
        if width:
            adj.set_value(width)
        else:
            width = adj.get_value()
        self.scrWidth = int(width)
        self.scrHeight = int(3 * self.scrWidth / 4) # Κeep the 4:3 aspect ratio
        
        # Fast scale all the thumbnails to make the change quickly visible
        old_pixbufs = self.imagetypes.values()
        for row in self.cstore:
            if row[C_PIXBUF] in old_pixbufs:
                ctype = row[C_INSTANCE].type
                cur_w = self.imagetypes[ctype].get_width()
                cur_h = self.imagetypes[ctype].get_height()
                if not (cur_w == self.scrWidth and cur_h == self.scrHeight):
                    new_thumb = row[C_PIXBUF].scale_simple(self.scrWidth, self.scrHeight, gtk.gdk.INTERP_NEAREST)
                    self.imagetypes[ctype] = new_thumb
                row[C_PIXBUF] = self.imagetypes[ctype]
            else:
                new_thumb = row[C_PIXBUF].scale_simple(self.scrWidth, self.scrHeight, gtk.gdk.INTERP_NEAREST)
                row[C_PIXBUF] = new_thumb
        
        # Hack to remove the extra padding that remains after a 'zoom out'
        self.cview.set_resize_mode(gtk.RESIZE_IMMEDIATE)
        self.cview.get_cells()[1].set_fixed_size(-1, -1)
        self.cview.check_resize()
    
    
    def scrIncreaseSize(self, widget):
        # Increase the size of screenshots by 2 pixels in width
        adj = self.get('iconsSizeAdjustment')
        adj.set_value(adj.get_value() + 15)
        self.reload_imagetypes()


    def scrDecreaseSize(self, widget):
        # Decrease the size of screenshots by 2 pixels in width
        adj = self.get('iconsSizeAdjustment')
        adj.set_value(adj.get_value() - 15)
        self.reload_imagetypes()
    
    
    def contextMenuPopup(self, widget, event):
        clicked = widget.get_path_at_pos(int(event.x), int(event.y))

        if event.button == 3:
            if widget is self.cview:
                selection = widget
                selected = widget.get_selected_items()
            else:
                selection = widget.get_selection()
                selected = selection.get_selected_rows()[1]
                if clicked:
                    clicked = clicked[0]

            if clicked:
                if not clicked in selected:
                    selection.unselect_all()
                    selection.select_path(clicked)
            else:
                selection.unselect_all()

            if widget is self.cview:
                menu = self.get('clients').get_submenu()
            
            menu.popup(None, None, None, event.button, event.time)
            menu.show()
            return True


    def on_clients_selection_changed(self, widget=None):
        selected = self.getSelectedClients()
        single_client = False
        if len(selected) == 1:
            single_client = True
        self.get('miClientProperties').set_sensitive(single_client)
        self.get('tb_client_properties').set_sensitive(single_client)
        
        if len(selected) > 0:
            self.get('miAddToGroup').set_sensitive(True)
            if not self.isDefaultGroupSelected():
                self.get('miRemoveFromGroup').set_sensitive(True)
            else:
                self.get('miRemoveFromGroup').set_sensitive(False)
        else:
            self.get('miAddToGroup').set_sensitive(False)
            self.get('miRemoveFromGroup').set_sensitive(False)
        
        if len(selected) > 1:
            self.get('statusbar_label').set_text(_('%d clients selected' % len(selected)))
        else:
            self.get('statusbar_label').set_text('')


    def execOnClients(self, command, clients=[], reply=None,
            mode=EM_SESSION_OR_SYSTEM, handles=[], warning='', params=None):
        '''reply should be a method in which the result will be sent'''
        if params is None:
            params = []
        if len(self.cstore) == 0:
            # print 'No clients'
            return False
        
        if isinstance(command, list) and len(command) > 0:
            command = '%s %s' %(command[0], ' '.join([pipes.quote(str(x)) for x in command[1:]]))
                    
        if (clients != [] or handles != []) and warning != '':
            if self.warnDlgPopup(warning) == False:
                return
        if clients == [] and handles != []:
            for handle in handles:
                cmd = self.daemon.command(handle, unicode(command))
                if reply:
                    cmd.addCallback(lambda re, h=handle, p=params: reply(h, re, *p))
                    cmd.addErrback(lambda err: self.printErrors("when executing command %s on client %s: %s" %(command,handle, err)))

        for client in clients:
            sent = False
            for em in mode:
                if em == EM_SESSION_ONLY:
                    handle = client[C_SESSION_HANDLE]
                elif em == EM_SYSTEM_ONLY:
                    handle = client[C_INSTANCE].hsystem
                else: # em == EM_EXIT_IF_SENT
                    if sent:
                        break
                    else:
                        continue
                if handle == '':
                    continue
                sent = True
                cmd = self.daemon.command(handle, unicode(command))
                if reply:
                    cmd.addCallback(lambda re, h=handle, p=params: reply(h, re, *p))
                    cmd.addErrback(lambda err: self.printErrors("when executing command %s on client %s: %s" %(command,handle, err)))


    def execOnSelectedClients(self, command, reply=None,
            mode=EM_SESSION_OR_SYSTEM, warn=''):
        clients = self.getSelectedClients()
        if len(clients) == 0: # No client selected, send the command to all
            clients = self.cstore
        else: # Show the warning only when no clients are selected
            warn = ''
        self.execOnClients(command, clients, reply, mode, warning=warn)


    def warnDlgPopup(self, warning):
        dlg = self.get('executionwarning')
        dlg.set_property('text', warning)
        resp = dlg.run()
        dlg.set_property('text', '')
        dlg.hide()
        # -8 is the response id of the "Yes" button (-9 for the "No")
        if resp == -8:
            return True
        else:
            return False


    def printErrors(self, error):
        print '  **Twisted error:', error
        return



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###########################################################################
# Command execution.
#
# Copyright (C) 2010 Fotis Tsamis <ftsamis@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FINESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# On Debian GNU/Linux systems, the complete text of the GNU General
# Public License can be found in `/usr/share/common-licenses/GPL".
###########################################################################

import gtk
import os

from epoptes.common import config


wTree = gtk.Builder()
get = lambda obj: wTree.get_object(obj)
store = gtk.ListStore(str)

def startExecuteCmdDlg():
    """Show the execute commands dialog and return the inserted command.

    If the dialog was closed, return an empty string.
    """
    wTree.add_from_file('executeCommand.ui')
    dlg = get('execDialog')
    combo = get('combobox')
    entry = combo.child
    completion = get('entrycompletion')
    entry.set_completion(completion)
    completion.set_model(store)
    entry.set_activates_default(True)
    combo.set_model(store)
    combo.set_text_column(0)
    
    entry.connect('changed', text_changed)
    
    combo = get('combobox')
    store.clear()
    
    for cmd in config.history:
        store.append([cmd])
    
    cmd = ''
    reply = dlg.run()
    if reply == 1:
        cmd = combo.child.get_text().strip()
        if cmd in config.history:
            config.history.remove(cmd)
        config.history.insert(0, cmd)
        config.write_history()
    dlg.destroy()
    
    return cmd

def text_changed(editable):
    if editable.get_text().strip():
        get('execute').set_sensitive(True)
    else:
        get('execute').set_sensitive(False)



LICENSE_TEXT = '''Epoptes is free software: you can redistribute it and/or 
modify it under the terms of the GNU General Public License as published by the 
Free Software Foundation, either version 3 of the License, or any later version. 

Epoptes is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
PARTICULAR PURPOSE. See the GNU General Public License for more details. 

You should have received a copy of the GNU General Public License along with 
Epoptes. If not, see <http://www.gnu.org/licenses/>.'''

__version__ = "0.5.10" # Automatically updated by mkdst
__license__ = "GPL"



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###########################################################################
# Message sending.
#
# Copyright (C) 2010 Fotis Tsamis <ftsamis@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FINESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# On Debian GNU/Linux systems, the complete text of the GNU General
# Public License can be found in `/usr/share/common-licenses/GPL".
###########################################################################

import gtk
import os
import pygtk

from epoptes.common import config

pygtk.require('2.0')

wTree = gtk.Builder()
get = lambda obj: wTree.get_object(obj)

def startSendMessageDlg():
    """
    Retrieve dialog window from glade format and
    according to type of message requested to send
    Returns: a 2-tuple containing the message text
    and the message type.
    """

    wTree.add_from_file('sendMessage.ui')
    dlg = get('sendMessageDialog')
    
    textView = get('Message')
    title_entry = get('title_entry')
    title_entry.set_text(config.settings.get('GUI', 'messages_default_title'))
    use_markup_toggle = get('use_markup_toggle')
    use_markup_toggle.set_active(config.settings.getboolean('GUI', 'messages_use_markup'))
    
    reply = dlg.run()
    msg = ()
    
    if reply == 1:
        buf = textView.get_buffer()
        s = buf.get_start_iter()
        e = buf.get_end_iter()
        text = textView.get_buffer().get_text(s,e)

        title = title_entry.get_text().strip()
        
        use_markup = use_markup_toggle.get_active()
        
        msg = (text, title, use_markup)

        config.settings.set('GUI', 'messages_default_title', title)
        config.settings.set('GUI', 'messages_use_markup', str(use_markup))
    
        f = open(os.path.expanduser('~/.config/epoptes/settings'), 'w')
        config.settings.write(f)
        f.close()
    # Hide dialog after any kind of function
    dlg.hide()

    # Return the command to be executed on clients
    return msg



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###########################################################################
# Common constants.
#
# Copyright (C) 2010 Fotis Tsamis <ftsamis@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FINESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# On Debian GNU/Linux systems, the complete text of the GNU General
# Public License can be found in `/usr/share/common-licenses/GPL".
###########################################################################

# Warn users to update their chroots if they have a lower epoptes-client version 
# than this
COMPATIBILITY_VERSION = [0, 5]

# ['ltsp123', '00-1b-24-89-65-d6', '127.0.0.1:46827', '10.160.31.126:44920', 
#  'thin', 'user3', <gtk.gdk.Pixbuf>, '10.160.31.123', 'user (ltsp123)']
C_LABEL = 0
C_PIXBUF = 1
C_INSTANCE = 2
C_SESSION_HANDLE = 3
# [label, <gtk.gdk.Pixbuf>, <Client instance>, username]
G_LABEL = 0
G_INSTANCE = 1

# Execution Modes are used in execOnClients
EM_SESSION_ONLY = 0
EM_SYSTEM_ONLY = 1
EM_EXIT_IF_SENT = 2
EM_SESSION = [EM_SESSION_ONLY]
EM_SESSION_AND_SYSTEM = [EM_SESSION_ONLY, EM_SYSTEM_ONLY]
EM_SESSION_OR_SYSTEM = [EM_SESSION_ONLY, EM_EXIT_IF_SENT, EM_SYSTEM_ONLY]
EM_SYSTEM_OR_SESSION = [EM_SYSTEM_ONLY, EM_EXIT_IF_SENT, EM_SESSION_ONLY]
EM_SYSTEM_AND_SESSION = [EM_SYSTEM_ONLY, EM_SESSION_ONLY]
EM_SYSTEM = [EM_SYSTEM_ONLY]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###########################################################################
# BASH plex.
#
# Copyright (C) 2010 Fotis Tsamis <ftsamis@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FINESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# On Debian GNU/Linux systems, the complete text of the GNU General
# Public License can be found in `/usr/share/common-licenses/GPL".
###########################################################################

import os
import uuid

from twisted.internet import reactor, protocol, defer, error

try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO

import exchange


class DelimitedBashReceiver(protocol.Protocol):
    """
    Send bash commands followed by "\necho <delimiter>",
    buffer responses until the given delimiter is found.
    """

    # Template into which per-command delimiters are
    # inserted. Should include trailing newline.
    delimiterTemplate = "__delimiter__%s__\n"

    def __init__(self):
        self.currentDelimiters = []
        self.buffer = StringIO()
        self.pingTimer = None
        self.pingTimeout = None
        self.timedOut = False


    def getDelimiter(self):
        """
        Generate a per-command delimiter. We could
        use a sequence number here instead, but I'm
        not sure we could do any more useful error
        handling with it.
        """
        return str(uuid.uuid4())


    def command(self, cmd, delimiter=None):
        """
        Send a command. We don't do any state-checking
        here. If another command is in progress, it'll
        probably be okay, but if serialization is required,
        it should be done elsewhere.
        """

        if delimiter is None:
            delimiter = self.delimiterTemplate % self.getDelimiter()

        d = defer.Deferred()

        self.currentDelimiters.append((delimiter, d))

        delimitedCommand = "%s\necho %s" % (
            cmd, delimiter)

        # TODO: check the python's debug logging implementation
        # print "Sending:", delimitedCommand,
        self.transport.write(delimitedCommand)

        return d


    def connectionMade(self):
        peer = self.transport.getPeer()
        self.handle = u"%s:%s" % (peer.host, peer.port)
        print "Connected:", self.handle
        
        d = self.command(self.factory.startupCommands.encode("utf-8"))
        
        def forwardConnection(result):
            exchange.clientConnected(self.handle, self)
            self.pingTimer = reactor.callLater(self.factory.pingInterval, self.ping)
        
        def killConnection(error):
            print "Error: Could not send the startup functions to the client:", error
            self._loseConnection()
        
        d.addCallback(forwardConnection)
        d.addErrback(killConnection)


    def connectionLost(self, reason):
        print "Connection lost:", self.handle
        
        try: self.pingTimeout.cancel()
        except Exception: pass

        try: self.pingTimer.cancel()
        except Exception: pass

        if self.handle in exchange.knownClients:
            exchange.clientDisconnected(self.handle)


    def dataReceived(self, data):
        self.buffer.seek(0, os.SEEK_END)
        self.buffer.write(data)

        if not self.currentDelimiters:
            return

        (delimiter, d) = self.currentDelimiters[0]
        #print "Searching for delimiter:", delimiter

        # Optimize for large buffers by not searching the whole thing every time
        searchLength = len(data) + len(delimiter)

        self.buffer.seek(-searchLength, os.SEEK_END)

        searchStr = self.buffer.read()
        searchPos = searchStr.find(delimiter)
        if searchPos != -1:
            #print "Found delimiter:", delimiter

            # Two steps here is correct! If the delimiter was received in the
            # first packet, then the searchLength is greater than the buffer
            # length, and doing this seek in one step gives the wrong answer.
            self.buffer.seek(-searchLength, os.SEEK_END)
            self.buffer.seek(searchPos, os.SEEK_CUR)
            pos = self.buffer.tell()

            self.buffer.seek(0)
            response = self.buffer.read(pos)

            # Throw away the delimiter
            self.buffer.read(len(delimiter))

            newBuffer = StringIO()
            newBuffer.write(self.buffer.read())
            self.buffer = newBuffer

            self.currentDelimiters.pop(0)
            d.callback(response)

            self.checkForFurtherResponses()


    def checkForFurtherResponses(self):
        # See if there are more responses in the buffer. 
        # The theory here is that if we got one already, we have less than one
        # packet of data left in the buffer, so reading it all isn't a big deal

        self.buffer.seek(0)
        rest = self.buffer.read()

        while self.currentDelimiters:
            (delimiter, d) = self.currentDelimiters[0]
            try:
                response, rest = rest.split(delimiter)
            except ValueError:
                break
                        
            self.currentDelimiters.pop(0)
            d.callback(response)
                
        newBuffer = StringIO()
        newBuffer.write(rest)
        self.buffer = newBuffer


    def ping(self):
        self.command('ping').addCallback(self.pingResponse)
        self.pingTimeout = reactor.callLater(self.factory.pingTimeout, 
                                             self.pingTimedOut)

    def pingResponse(self, _):
        # Responses that arrive after a client has timed out, mean a "reconnect"
        if self.timedOut:
            print "Reconnected:", self.handle
            exchange.clientReconnected(self.handle)
            self.timedOut = False
        else:
            self.pingTimeout.cancel()
        self.pingTimer = reactor.callLater(self.factory.pingInterval, self.ping)


    def pingTimedOut(self):
        print "Ping timeout:", self.handle
        self.timedOut = True
        exchange.clientTimedOut(self.handle)


class DelimitedBashReceiverFactory(protocol.ServerFactory):
    protocol = DelimitedBashReceiver
    
    pingInterval = 10
    pingTimeout = 10
    
    startupCommands = ''



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###########################################################################
# Utility functions to get a user's localized XDG directories.
#
# Copyright (C) 2010 Alkis Georgopoulos <alkisg@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FINESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# On Debian GNU/Linux systems, the complete text of the GNU General
# Public License can be found in `/usr/share/common-licenses/GPL".
###########################################################################

import os
import pwd
import shlex
import gettext

domain = "xdg-user-dirs"
gettext.bindtextdomain(domain)
gettext.textdomain(domain)
T_ = gettext.gettext


def read_config_file(path):
    """Return the contents of a shell-like file in a string list.

    If the file doesn't exist or isn't readable, return an empty list.
    Also strip all comments, if any.
    """

    if not os.path.isfile(path):
        return []

    try:
        f = open(path, 'r')
        contents = f.read()
        f.close()
        return shlex.split(contents, True)
    except:
        return []


def read_vars_file(path):
    """Return a dictionary read from a shell vars file.
    """
    
    contents = read_config_file(path)
    try:
        # TODO: maybe return at least all the valid pairs?
        return dict(v.split('=') for v in contents)
    except:
        return {}


def localized_names(lang):
    """Return a dictionary with the translations of XDG directories for lang.
    """

    # If lang is unset, the current locale is used
    if lang:
        oldlang = os.environ['LANG']
        os.environ['LANG'] = lang

    # Directory names are from xdg-user-dirs/translate.c
    result = {
        'Applications': T_('Applications'),
        'Desktop': T_('Desktop'),
        'Documents': T_('Documents'),
        'Download': T_('Download'),
        'Downloads': T_('Downloads'),
        'Movies': T_('Movies'),
        'Music': T_('Music'),
        'Photos': T_('Photos'),
        'Pictures': T_('Pictures'),
        'Projects': T_('Projects'),
        'Public': T_('Public'),
        'Share': T_('Share'),
        'Templates': T_('Templates'),
        'Videos': T_('Videos')
        }

    if lang:
        os.environ['LANG'] = oldlang

    return result


def localize_dir(path, lnames):
    """Localize a default XDG dir like e.g. Documents/Pictures.
    """

    components = path.split('/')
    for i in range(len(components)):
        if components[i] in lnames:
            components[i] = lnames[components[i]]

    return '/'.join(components)


def default_dirs(lang):
    """Return the default XDG dirs for lang.

    These are read from /etc/xdg/user-dirs.defaults.
    """

    # Map from default xdg dirs to normal lowercase vars
    d2n = {
        'APPLICATIONS': 'Applications',
        'DESKTOP': 'Desktop',
        'DOCUMENTS': 'Documents',
        'DOWNLOAD': 'Download',
        'DOWNLOADS': 'Downloads',
        'MOVIES': 'Movies',
        'MUSIC': 'Music',
        'PHOTOS': 'Photos',
        'PICTURES': 'Pictures',
        'PROJECTS': 'Projects',
        'PUBLIC': 'Public',
        'PUBLICSHARE': 'Public',
        'SHARE': 'Share',
        'TEMPLATES': 'Templates',
        'VIDEOS': 'Videos'}

    lnames = localized_names(lang)
    ddirs = read_vars_file('/etc/xdg/user-dirs.defaults')
    result = lnames
    for key, val in ddirs.iteritems():
        if key in d2n:
            result[d2n[key]] = localize_dir(val, lnames)

    return result


def user_dirs(home):
    """Return the user XDG dirs for the user at "home".

    These are read from ~/.config/user-dirs.dirs.
    """

    # Map from user xdg dirs to normal lowercase vars
    u2n = {
        'XDG_APPLICATIONS_DIR': 'Applications',
        'XDG_DESKTOP_DIR': 'Desktop',
        'XDG_DOCUMENTS_DIR': 'Documents',
        'XDG_DOWNLOAD_DIR': 'Download',
        'XDG_DOWNLOADS_DIR': 'Downloads',
        'XDG_MOVIES_DIR': 'Movies',
        'XDG_MUSIC_DIR': 'Music',
        'XDG_PHOTOS_DIR': 'Photos',
        'XDG_PICTURES_DIR': 'Pictures',
        'XDG_PROJECTS_DIR': 'Projects',
        'XDG_PUBLIC_DIR': 'Public',
        'XDG_PUBLICSHARE_DIR': 'Public',
        'XDG_SHARE_DIR': 'Share',
        'XDG_TEMPLATES_DIR': 'Templates',
        'XDG_VIDEOS_DIR': 'Videos'}

    udirs = read_vars_file("%s/.config/user-dirs.dirs" % home)
    result = {}
    for key, val in udirs.iteritems():
        if key in u2n:
            result[u2n[key]] = val

    return result


def user_lang(home):
    """Return the user language for XDG directories, if set.
    """

    contents = read_config_file("%s/.config/user-dirs.locale" % home)
    if len(contents) == 1:
        return contents[0]
    else:
        return ''
    

def system_lang():
    """Return the default system locale which is applied to new users.
        
    It is read from /etc/default/locale.
    """

    flocale = read_vars_file('/etc/default/locale')
    return flocale.get('LANG', '')


def expand_home(dic, home):
    """
    """
#    for key in result.keys():
#        if result[key].startswith('$HOME'):
#            result[key] = result[key].replace('$HOME', home)
#
#    dict.get(key, default) will be useful
#    key in dict also
#    pop(key, default)
#    setdefault(key, default)
#    update(other_dict)
#    dict | other → union
    pass

    
def get_xdg_dirs(user):
    """Return a dictionary with the xdg dirs for that user.

    If any or all of that user's xdg dirs aren't set, return the default
    xdg dirs based on the default system locale.
    Raise an error if the user doesn't exist.
    """

    # The caller is responsible to catch any exceptions raised by these:
    if type(user) is int:
        pwentry = pwd.getpwuid(user)
    else:
        pwentry = pwd.getpwnam(user)

    # Do a basic check that no system user was selected
    if pwentry.pw_uid < 999:
        raise Exception('Tried to get the xdg dirs for a system user')    

    home = pwentry.pw_dir
    
    # If a ~/.config/user-dirs.locale exists, use it
    lang = user_lang(home)

    # Otherwise use the default system language, or '' for the current lang
    if not lang:
        lang = system_lang()

    # Get the default and the user specified directories
    ddirs = default_dirs(lang)
    udirs = user_dirs(home)

    # And merge them into result, while replacing $HOME when necessary
    result = ddirs
    result.update(udirs)
    for key, val in result.iteritems():
        if val.startswith('$HOME'):
            result[key] = val.replace('$HOME', home, 1)
        elif not val.startswith('/'):
            result[key] = home + '/' + val

    return result


# Allow xdg_dirs to be ran from the command line
if __name__ == '__main__':
    import sys

    if len(sys.argv) > 1:
        user = sys.argv[1]
    else:
        user = os.environ["USER"]

    for key, val in get_xdg_dirs(user).iteritems():
        print "%s=%s" % (key, val)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###########################################################################
# About dialog.
#
# Copyright (C) 2010 Fotis Tsamis <ftsamis@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FINESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# On Debian GNU/Linux systems, the complete text of the GNU General
# Public License can be found in `/usr/share/common-licenses/GPL".
###########################################################################

import gtk
import pygtk

from epoptes import __version__

class About:
    def __init__(self):
        self.wTree = gtk.Builder()
        self.wTree.add_from_file('about_dialog.ui')
        self.wTree.connect_signals(self)
        self.get = self.wTree.get_object
        
        self.dialog = self.get('aboutdialog')
        logo = gtk.gdk.pixbuf_new_from_file_at_size(
            '../icons/hicolor/scalable/apps/epoptes.svg', 64, 64)
        self.dialog.set_logo(logo)
        self.dialog.set_version(__version__)
        self.dialog.set_translator_credits(_("translator-credits"))
        self.dialog.set_artists(["Andrew Wedderburn (application icon)"])
    
    def run(self):
        self.dialog.run()
        self.dialog.destroy()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###########################################################################
# Remote assistance.
#
# Copyright (C) 2010 Alkis Georgopoulos <alkisg@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FINESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# On Debian GNU/Linux systems, the complete text of the GNU General
# Public License can be found in `/usr/share/common-licenses/GPL".
###########################################################################

import gtk
import pygtk
import subprocess

class RemoteAssistance:
    def __init__(self):
        self.wTree = gtk.Builder()
        self.wTree.add_from_file('remote_assistance.ui')
        self.wTree.connect_signals(self)
        self.get = self.wTree.get_object
    
    def run(self):    
        dlg = self.get('remote_assistance_dialog')
        if self.get('sb_assist_port').get_value() == 0:
            self.get('sb_assist_port').set_value(5500)
        reply = dlg.run()
        if reply == 1:
            ip = self.get('rem_assist_ip').get_text().strip()
            port = self.get('sb_assist_port').get_value()
            if self.get('cb_assist_type').get_active() == 1:
                # Unfortunately double quoting is needed when a parameter 
                # contains spaces. That might change in the future, 
                # see http://www.sudo.ws/sudo/bugs/show_bug.cgi?id=413
                # Fortunately, sh -c 'ls' works even if the quotes there are 
                # wrong. :)
                subprocess.Popen(['sh', '-c', ("""TERM=xterm socat """ +
                    """SYSTEM:"sleep 1; exec screen -xRR ra",pty,stderr """ +
                    """tcp:%s:%d & exec xterm -e screen -l -S ra""") %
                    (ip, port)])
            else:
                subprocess.Popen(['./epoptes-remote-assistance', "%s:%d" % (ip, port)])
        dlg.destroy()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###########################################################################
# Get client properties.
#
# Copyright (C) 2010 Fotis Tsamis <ftsamis@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FINESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# On Debian GNU/Linux systems, the complete text of the GNU General
# Public License can be found in `/usr/share/common-licenses/GPL".
###########################################################################

import gtk
import pygtk

from epoptes.common.constants import *

class ClientInformation:
    def __init__(self, selected, execute):
        self.wTree = gtk.Builder()
        self.wTree.add_from_file('client_information.ui')
        self.wTree.connect_signals(self)
        self.get = self.wTree.get_object
        self.selected = selected
        
        self.dlg = self.get('infodlg')
        self.edit_button = self.get('edit_alias_button')
        set = lambda wdg, txt: self.get(wdg).set_text(txt.strip())

        for client in selected:
            inst = client[C_INSTANCE]
            handle = inst.hsystem or client[C_SESSION_HANDLE]
            if handle:
                execute(handle, 'echo $RAM').addCallback(
                    lambda r: set('client_ram', r.strip()+' MB'))
                execute(handle, 'echo $CPU').addCallback(
                    lambda r: set('client_cpu', r))
                execute(handle, 'echo $VGA').addCallback(
                    lambda r: set('client_vga', r))
            set('client_alias', inst.alias)
            set('client_hostname', inst.hostname)
            set('client_mac', inst.mac)
            set('client_ip', handle.split(':')[0])
            set('client_type', inst.type)
            user = '--'
            if client[C_SESSION_HANDLE]:
                uname, realname = inst.users[client[C_SESSION_HANDLE]].values()
                user = uname
                if realname:
                    user += ' (%s)' %realname
            set('client_online_user', user)
            self.dlg.set_title(_('Properties of %s') %inst.get_name())
        
    def run(self):
        self.dlg.run()
        self.dlg.destroy()
        
    def on_edit_alias_clicked(self, widget):
        inst = self.selected[0][C_INSTANCE]
        edit_dlg = self.get('edit_alias_dialog')
        entry = self.get('alias_entry')
        entry.set_text(inst.alias)
        resp = edit_dlg.run()
        if resp == 1:
            inst.set_name(entry.get_text().strip())
            self.get('client_alias').set_text(inst.alias.strip())
        edit_dlg.hide()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###########################################################################
# Epoptesd.
#
# Copyright (C) 2010 Fotis Tsamis <ftsamis@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FINESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# On Debian GNU/Linux systems, the complete text of the GNU General
# Public License can be found in `/usr/share/common-licenses/GPL".
###########################################################################

import os
from zope.interface import implements

from twisted.python import usage
from twisted.plugin import IPlugin
from twisted.application.service import IServiceMaker
from twisted.application import internet, service

from epoptes.daemon import bashplex, guiplex
from epoptes.common import config
import grp
from OpenSSL import SSL


class Options(usage.Options):
    optParameters = [
        ("clientport", "p", 789, "Client Port"),
        ('pingInterval', 'i', 10),
        ('pingTimeout', 't', 10),
      ]


class ServerContextFactory:
    def getContext(self):
        ctx = SSL.Context(SSL.SSLv23_METHOD)
        ctx.use_certificate_file("/etc/epoptes/server.crt")
        ctx.use_privatekey_file("/etc/epoptes/server.key")
        return ctx


class ServiceMaker(object):
    implements(IServiceMaker, IPlugin)
    tapname = "epoptes"
    description = "Epoptes Daemon"
    options = Options

    def makeService(self, options):
        
        factory = bashplex.DelimitedBashReceiverFactory()
        factory.pingInterval=int(options['pingInterval'])
        factory.pingTimeout=int(options['pingTimeout'])
        factory.startupCommands = self.filterBash('/usr/share/epoptes/client-functions')

        if config.system['ENCRYPTION']:
            clientService = internet.SSLServer(int(config.system['PORT']),
                factory, ServerContextFactory())
        else:
            clientService = internet.TCPServer(int(config.system['PORT']),
                factory)

        gid = grp.getgrnam(config.system['SOCKET_GROUP'])[2]
        
        if not os.path.isdir(config.system['DIR']):
            #TODO: for some reason this does 0750 instead
            os.makedirs(config.system['DIR'], 02770)
        os.chmod(config.system['DIR'], 02770)
        os.chown(config.system['DIR'], -1, gid)

        guiService = internet.UNIXServer(
            "%s/epoptes.socket" % config.system['DIR'],
            guiplex.GUIFactory())

        topService = service.MultiService()
        topService.addService(clientService)
        topService.addService(guiService)

        return topService
    
    def filterBash(self, script):
        f = open(script)
        functions = f.readlines()
        f.close()
        
        result = ''
        
        for line in functions:
            if line.strip() != '' and line.strip()[0] == '#':
                continue
            result += line
        return result

serviceMaker = ServiceMaker()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###########################################################################
# This script needs python-distutils-extra, an extension to the standard
# distutils which provides i18n, icon support, etc.
# https://launchpad.net/python-distutils-extra
#
# Copyright (C) 2011 Alkis Georgopoulos <alkisg@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FINESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# On Debian GNU/Linux systems, the complete text of the GNU General
# Public License can be found in `/usr/share/common-licenses/GPL".
###########################################################################

from glob import glob
from distutils.version import StrictVersion

try:
    import DistUtilsExtra.auto
except ImportError:
    import sys
    print >> sys.stderr, 'To build epoptes you need https://launchpad.net/python-distutils-extra'
    sys.exit(1)

assert StrictVersion(DistUtilsExtra.auto.__version__) >= '2.4', 'needs DistUtilsExtra.auto >= 2.4'

import posixpath, re

def changelog_version(changelog="debian/changelog"):
    version = "dev"
    if posixpath.exists(changelog):
        head=open(changelog).readline()
        match = re.compile(".*\((.*)\).*").match(head)
        if match:
            version = match.group(1)

    return version

def subtract_files(a, b):
    res = set(a)
    for dir, files in b:
        res -= set(files)
    return list(res)

client_special_files=[
    ('/etc/xdg/autostart/',
        ['epoptes-client/epoptes-client.desktop']),
    ('/usr/sbin/',
        ['epoptes-client/epoptes-client']),
    ('/usr/share/ldm/rc.d/',
        ['epoptes-client/X50-client-env']),
    ]
client_usr_share_files=[
    ('/usr/share/epoptes-client/',
        subtract_files(glob('epoptes-client/*'), client_special_files))
    ]
server_special_files=[
    ('/usr/share/ltsp/plugins/ltsp-build-client/common/',
        ['data/040-epoptes-certificate'])];

DistUtilsExtra.auto.setup(
    name='epoptes',
    version = changelog_version(),
    description = 'Computer lab administration and monitoring tool',
    url = 'http://www.epoptes.org',
    license = 'GNU GPL v3',
    author = 'Fotis Tsamis',
    author_email = 'ftsamis@gmail.com',
    py_modules = ['twisted.plugins.epoptesd'],
    data_files = client_special_files + client_usr_share_files +
        server_special_files
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###########################################################################
# This will read lts.conf and enumerate the clients by MAC so they
# can be started up with Wake-On-Lan.
#
# Copyright (C) 2010 Fotis Tsamis <ftsamis@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FINESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# On Debian GNU/Linux systems, the complete text of the GNU General
# Public License can be found in `/usr/share/common-licenses/GPL".
###########################################################################

import re
import os

class ltsConf:
    
    def __init__(self):
        self.rmcomment = lambda string: string.split('#')[0].strip()
        self.getvar = lambda string: self.rmcomment(string).split('=', 1)[0].strip()
        self.getval = lambda string: self.rmcomment(string).split('=', 1)[1].strip()
        self.filename = '/var/lib/tftpboot/ltsp/i386/lts.conf'
        self.parse()
        
    def parse(self):
        if os.path.isfile(self.filename):
            file = open(self.filename)
            lines = file.readlines()
            file.close()
        else:
            lines = []
        
        self.content = lines#''.join(lines)
        self.clients = {}
        
        cursection = None
        for i in range(len(lines)):
            curline = lines[i].strip()
            
            if curline == '':
                continue
            
            elif curline[0] == '[':
                if re.match('\[..:..:..:..:..:..\]', curline):
                    cursection = self.rmcomment(curline).strip('[]').upper()
                    self.clients[cursection] = {}
                else:
                    cursection = None
            
            elif '=' in self.rmcomment(curline) and cursection != None:
                item = self.getvar(curline)
                value = self.getval(curline)
                self.clients[cursection][item] = value
    
    def write(self):
        data = self.content#.strip().readlines()
        f = open(self.filename,'w')#'/var/lib/tftpboot/ltsp/i386/lts.conf', 'w')
        
        cursection = None
        for i in range(len(data)):
            curline = data[i].strip()
            
            if curline == '':
                f.write('\n')
            
            elif curline[0] == '[':
                cursection = self.rmcomment(curline).strip('[]')
                if re.match('\[..:..:..:..:..:..\]', curline):
                    if cursection in self.clients:
                        f.write(data[i])
                    else:
                        cursection = None
                else:
                    cursection = 'ignored'
                    f.write(data[i])
            
            elif curline[0] == '#':
                f.write(data[i])
            
            elif '=' in self.rmcomment(curline) and cursection != None:
                if cursection == 'ignored':
                    f.write(data[i])
                elif self.getvar(curline) in self.clients[cursection]:
                    dat = data[i].replace(self.getval(curline), self.clients[cursection][self.getvar(curline)])
                    f.write(dat)
        
        f.close()
        self.parse()
    
    def getSavedClients(self):
        list = []
        for client, vars in self.clients.iteritems():
            if 'HOSTNAME' in vars:
                list.append(client)
        return list
    
    def isSaved(self, mac):
        if mac in self.getSavedClients():
            return True
        return False
        
    def saveClient(self, mac, hostname):
        if self.isSaved(mac):
            return
        #FIXME: section may exists
        f = open(self.filename, 'a')
        f.write('\n[%s]\n\tHOSTNAME=%s\n' %(mac, hostname))
        f.close()
        self.parse()
    
    def removeClient(self, mac):
        if isSaved(mac):
            del self.clients[mac]['HOSTNAME']
            self.write()
        
    def set(self, section, item, value):
        self.clients[section][item] = value
        self.write()
    
    def getHostname(self, mac):
        if self.isSaved(section):
            return self.clients[section]['HOSTNAME']
    
    
    # DELETE US
    def getItem(self, section, item):
        if self.itemExists(section, item):
            return self.clients[section][item]
        
    def addItem(self, section, item, value):
        pass
    
    def rmSection(self, name):
        if sectionExists(name):
            del self.clients[name]
            self.write()
    
    def getSections(self):
        return self.clients.keys()
    
    def sectionExists(self, section):
        if section in self.clients:
            return True
        return False
    
    def itemExists(self, section, item):
        if self.sectionExists(section) and item in self.clients[section]:
            return True
        return False
        
    
